"""
Routines for working with functions within the disassembled program.

This file also contains routines for working with library signatures (e.g.
FLIRT).

Each function consists of function chunks. At least one function chunk must be
present in the function definition - the function entry chunk. Other chunks are
called function tails. There may be several of them for a function.

A function tail is a continuous range of addresses. It can be used in the
definition of one or more functions. One function using the tail is singled out
and called the tail owner. This function is considered as 'possessing' the tail.
get_func() on a tail address will return the function possessing the tail. You
can enumerate the functions using the tail by using func_parent_iterator_t.

Each function chunk in the disassembly is represented as an "range" (a range of
addresses, see range.hpp for details) with characteristics.

A function entry must start with an instruction (code) byte."""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_funcs
else:
    import _ida_funcs

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_funcs.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

import ida_range
class dyn_stkpnt_array(object):
    r"""
    Proxy of C++ dynamic_wrapped_array_t< stkpnt_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_ida_funcs.dyn_stkpnt_array_data_get, doc=r"""data""")
    count = property(_ida_funcs.dyn_stkpnt_array_count_get, doc=r"""count""")

    def __init__(self, *args):
        r"""
        __init__(self, _data, _count) -> dyn_stkpnt_array

        @param _data: stkpnt_t *
        @param _count: size_t
        """
        _ida_funcs.dyn_stkpnt_array_swiginit(self, _ida_funcs.new_dyn_stkpnt_array(*args))

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_funcs.dyn_stkpnt_array___len__(self, *args)

    def __getitem__(self, *args) -> "stkpnt_t const &":
        r"""
        __getitem__(self, i) -> stkpnt_t const &

        @param i: size_t
        """
        return _ida_funcs.dyn_stkpnt_array___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: stkpnt_t const &
        """
        return _ida_funcs.dyn_stkpnt_array___setitem__(self, *args)

    __iter__ = ida_idaapi._bounded_getitem_iterator

    __swig_destroy__ = _ida_funcs.delete_dyn_stkpnt_array

# Register dyn_stkpnt_array in _ida_funcs:
_ida_funcs.dyn_stkpnt_array_swigregister(dyn_stkpnt_array)

class dyn_regvar_array(object):
    r"""
    Proxy of C++ dynamic_wrapped_array_t< regvar_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_ida_funcs.dyn_regvar_array_data_get, doc=r"""data""")
    count = property(_ida_funcs.dyn_regvar_array_count_get, doc=r"""count""")

    def __init__(self, *args):
        r"""
        __init__(self, _data, _count) -> dyn_regvar_array

        @param _data: regvar_t *
        @param _count: size_t
        """
        _ida_funcs.dyn_regvar_array_swiginit(self, _ida_funcs.new_dyn_regvar_array(*args))

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_funcs.dyn_regvar_array___len__(self, *args)

    def __getitem__(self, *args) -> "regvar_t const &":
        r"""
        __getitem__(self, i) -> regvar_t const &

        @param i: size_t
        """
        return _ida_funcs.dyn_regvar_array___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: regvar_t const &
        """
        return _ida_funcs.dyn_regvar_array___setitem__(self, *args)

    __iter__ = ida_idaapi._bounded_getitem_iterator

    __swig_destroy__ = _ida_funcs.delete_dyn_regvar_array

# Register dyn_regvar_array in _ida_funcs:
_ida_funcs.dyn_regvar_array_swigregister(dyn_regvar_array)

class dyn_range_array(object):
    r"""
    Proxy of C++ dynamic_wrapped_array_t< range_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_ida_funcs.dyn_range_array_data_get, doc=r"""data""")
    count = property(_ida_funcs.dyn_range_array_count_get, doc=r"""count""")

    def __init__(self, *args):
        r"""
        __init__(self, _data, _count) -> dyn_range_array

        @param _data: range_t *
        @param _count: size_t
        """
        _ida_funcs.dyn_range_array_swiginit(self, _ida_funcs.new_dyn_range_array(*args))

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_funcs.dyn_range_array___len__(self, *args)

    def __getitem__(self, *args) -> "range_t const &":
        r"""
        __getitem__(self, i) -> range_t

        @param i: size_t
        """
        return _ida_funcs.dyn_range_array___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: range_t const &
        """
        return _ida_funcs.dyn_range_array___setitem__(self, *args)

    __iter__ = ida_idaapi._bounded_getitem_iterator

    __swig_destroy__ = _ida_funcs.delete_dyn_range_array

# Register dyn_range_array in _ida_funcs:
_ida_funcs.dyn_range_array_swigregister(dyn_range_array)

class dyn_ea_array(object):
    r"""
    Proxy of C++ dynamic_wrapped_array_t< ea_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_ida_funcs.dyn_ea_array_data_get, doc=r"""data""")
    count = property(_ida_funcs.dyn_ea_array_count_get, doc=r"""count""")

    def __init__(self, *args):
        r"""
        __init__(self, _data, _count) -> dyn_ea_array

        @param _data: unsigned long long *
        @param _count: size_t
        """
        _ida_funcs.dyn_ea_array_swiginit(self, _ida_funcs.new_dyn_ea_array(*args))

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_funcs.dyn_ea_array___len__(self, *args)

    def __getitem__(self, *args) -> "unsigned long long const &":
        r"""
        __getitem__(self, i) -> unsigned long long const &

        @param i: size_t
        """
        return _ida_funcs.dyn_ea_array___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: unsigned long long const &
        """
        return _ida_funcs.dyn_ea_array___setitem__(self, *args)

    __iter__ = ida_idaapi._bounded_getitem_iterator

    __swig_destroy__ = _ida_funcs.delete_dyn_ea_array

# Register dyn_ea_array in _ida_funcs:
_ida_funcs.dyn_ea_array_swigregister(dyn_ea_array)

class dyn_regarg_array(object):
    r"""
    Proxy of C++ dynamic_wrapped_array_t< regarg_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_ida_funcs.dyn_regarg_array_data_get, doc=r"""data""")
    count = property(_ida_funcs.dyn_regarg_array_count_get, doc=r"""count""")

    def __init__(self, *args):
        r"""
        __init__(self, _data, _count) -> dyn_regarg_array

        @param _data: regarg_t *
        @param _count: size_t
        """
        _ida_funcs.dyn_regarg_array_swiginit(self, _ida_funcs.new_dyn_regarg_array(*args))

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_funcs.dyn_regarg_array___len__(self, *args)

    def __getitem__(self, *args) -> "regarg_t const &":
        r"""
        __getitem__(self, i) -> regarg_t

        @param i: size_t
        """
        return _ida_funcs.dyn_regarg_array___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: regarg_t const &
        """
        return _ida_funcs.dyn_regarg_array___setitem__(self, *args)

    __iter__ = ida_idaapi._bounded_getitem_iterator

    __swig_destroy__ = _ida_funcs.delete_dyn_regarg_array

# Register dyn_regarg_array in _ida_funcs:
_ida_funcs.dyn_regarg_array_swigregister(dyn_regarg_array)


def free_regarg(*args) -> "void":
    r"""
    free_regarg(v)

    @param v: regarg_t *
    """
    return _ida_funcs.free_regarg(*args)
class regarg_t(object):
    r"""
    Proxy of C++ regarg_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    reg = property(_ida_funcs.regarg_t_reg_get, _ida_funcs.regarg_t_reg_set, doc=r"""reg""")
    type = property(_ida_funcs.regarg_t_type_get, _ida_funcs.regarg_t_type_set, doc=r"""type""")
    name = property(_ida_funcs.regarg_t_name_get, _ida_funcs.regarg_t_name_set, doc=r"""name""")

    def __init__(self, *args):
        r"""
        __init__(self) -> regarg_t
        __init__(self, r) -> regarg_t

        @param r: regarg_t const &
        """
        _ida_funcs.regarg_t_swiginit(self, _ida_funcs.new_regarg_t(*args))
    __swig_destroy__ = _ida_funcs.delete_regarg_t

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: regarg_t &
        """
        return _ida_funcs.regarg_t_swap(self, *args)

# Register regarg_t in _ida_funcs:
_ida_funcs.regarg_t_swigregister(regarg_t)

class func_t(ida_range.range_t):
    r"""
    Proxy of C++ func_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    flags = property(_ida_funcs.func_t_flags_get, _ida_funcs.func_t_flags_set, doc=r"""flags""")
    r"""
    Function flags
    """

    def is_far(self, *args) -> "bool":
        r"""
        is_far(self) -> bool
        Is a far function?
        """
        return _ida_funcs.func_t_is_far(self, *args)

    def does_return(self, *args) -> "bool":
        r"""
        does_return(self) -> bool
        Does function return?
        """
        return _ida_funcs.func_t_does_return(self, *args)

    def analyzed_sp(self, *args) -> "bool":
        r"""
        analyzed_sp(self) -> bool
        Has SP-analysis been performed?
        """
        return _ida_funcs.func_t_analyzed_sp(self, *args)

    def need_prolog_analysis(self, *args) -> "bool":
        r"""
        need_prolog_analysis(self) -> bool
        Needs prolog analysis?
        """
        return _ida_funcs.func_t_need_prolog_analysis(self, *args)
    frame = property(_ida_funcs.func_t_frame_get, _ida_funcs.func_t_frame_set, doc=r"""frame""")
    r"""
    netnode id of frame structure - see frame.hpp
    """
    frsize = property(_ida_funcs.func_t_frsize_get, _ida_funcs.func_t_frsize_set, doc=r"""frsize""")
    r"""
    size of local variables part of frame in bytes. If FUNC_FRAME is set and fpd==0,
    the frame pointer (EBP) is assumed to point to the top of the local variables
    range.
    """
    frregs = property(_ida_funcs.func_t_frregs_get, _ida_funcs.func_t_frregs_set, doc=r"""frregs""")
    r"""
    size of saved registers in frame. This range is immediately above the local
    variables range.
    """
    argsize = property(_ida_funcs.func_t_argsize_get, _ida_funcs.func_t_argsize_set, doc=r"""argsize""")
    r"""
    number of bytes purged from the stack upon returning
    """
    fpd = property(_ida_funcs.func_t_fpd_get, _ida_funcs.func_t_fpd_set, doc=r"""fpd""")
    r"""
    frame pointer delta. (usually 0, i.e. realBP==typicalBP) use update_fpd() to
    modify it.
    """
    color = property(_ida_funcs.func_t_color_get, _ida_funcs.func_t_color_set, doc=r"""color""")
    r"""
    user defined function color
    """
    pntqty = property(_ida_funcs.func_t_pntqty_get, _ida_funcs.func_t_pntqty_set, doc=r"""pntqty""")
    r"""
    number of SP change points
    """
    points = property(_ida_funcs.func_t_points_get, _ida_funcs.func_t_points_set, doc=r"""points""")
    r"""
    array of SP change points. use ...stkpnt...() functions to access this array.
    """
    regvarqty = property(_ida_funcs.func_t_regvarqty_get, _ida_funcs.func_t_regvarqty_set, doc=r"""regvarqty""")
    r"""
    number of register variables (-1-not read in yet) use find_regvar() to read
    register variables
    """
    regvars = property(_ida_funcs.func_t_regvars_get, _ida_funcs.func_t_regvars_set, doc=r"""regvars""")
    r"""
    array of register variables. this array is sorted by: start_ea. use
    ...regvar...() functions to access this array.
    """
    regargqty = property(_ida_funcs.func_t_regargqty_get, _ida_funcs.func_t_regargqty_set, doc=r"""regargqty""")
    r"""
    number of register arguments. During analysis IDA tries to guess the register
    arguments. It stores store the guessing outcome in this field. As soon as it
    determines the final function prototype, regargqty is set to zero.
    """
    regargs = property(_ida_funcs.func_t_regargs_get, _ida_funcs.func_t_regargs_set, doc=r"""regargs""")
    r"""
    unsorted array of register arguments. use ...regarg...() functions to access
    this array. regargs are destroyed when the full function type is determined.
    """
    tailqty = property(_ida_funcs.func_t_tailqty_get, _ida_funcs.func_t_tailqty_set, doc=r"""tailqty""")
    r"""
    number of function tails
    """
    tails = property(_ida_funcs.func_t_tails_get, _ida_funcs.func_t_tails_set, doc=r"""tails""")
    r"""
    array of tails, sorted by ea. use func_tail_iterator_t to access function tails.
    """
    owner = property(_ida_funcs.func_t_owner_get, _ida_funcs.func_t_owner_set, doc=r"""owner""")
    r"""
    the address of the main function possessing this tail
    """
    refqty = property(_ida_funcs.func_t_refqty_get, _ida_funcs.func_t_refqty_set, doc=r"""refqty""")
    r"""
    number of referers
    """
    referers = property(_ida_funcs.func_t_referers_get, _ida_funcs.func_t_referers_set, doc=r"""referers""")
    r"""
    array of referers (function start addresses). use func_parent_iterator_t to
    access the referers.
    """

    def __init__(self, *args):
        r"""
        __init__(self, start=0, end=0, f=0) -> func_t

        @param start: ea_t
        @param end: ea_t
        @param f: flags64_t
        """
        _ida_funcs.func_t_swiginit(self, _ida_funcs.new_func_t(*args))

    def __get_points__(self, *args) -> "dynamic_wrapped_array_t< stkpnt_t >":
        r"""
        __get_points__(self) -> dyn_stkpnt_array
        """
        return _ida_funcs.func_t___get_points__(self, *args)

    def __get_regvars__(self, *args) -> "dynamic_wrapped_array_t< regvar_t >":
        r"""
        __get_regvars__(self) -> dyn_regvar_array
        """
        return _ida_funcs.func_t___get_regvars__(self, *args)

    def __get_tails__(self, *args) -> "dynamic_wrapped_array_t< range_t >":
        r"""
        __get_tails__(self) -> dyn_range_array
        """
        return _ida_funcs.func_t___get_tails__(self, *args)

    def __get_referers__(self, *args) -> "dynamic_wrapped_array_t< ea_t >":
        r"""
        __get_referers__(self) -> dyn_ea_array
        """
        return _ida_funcs.func_t___get_referers__(self, *args)

    def __get_regargs__(self, *args) -> "dynamic_wrapped_array_t< regarg_t >":
        r"""
        __get_regargs__(self) -> dyn_regarg_array
        """
        return _ida_funcs.func_t___get_regargs__(self, *args)

    points = property(__get_points__)
    regvars = property(__get_regvars__)
    tails = property(__get_tails__)
    referers = property(__get_referers__)
    regargs = property(__get_regargs__)


    def addresses(self):
        r"""
        Alias for func_item_iterator_t(self).addresses()
        """
        yield from func_item_iterator_t(self).addresses()


    def code_items(self):
        r"""
        Alias for func_item_iterator_t(self).code_items()
        """
        yield from func_item_iterator_t(self).code_items()


    def data_items(self):
        r"""
        Alias for func_item_iterator_t(self).data_items()
        """
        yield from func_item_iterator_t(self).data_items()


    def head_items(self):
        r"""
        Alias for func_item_iterator_t(self).head_items()
        """
        yield from func_item_iterator_t(self).head_items()


    def not_tails(self):
        r"""
        Alias for func_item_iterator_t(self).not_tails()
        """
        yield from func_item_iterator_t(self).not_tails()


    def __iter__(self):
        r"""
        Alias for func_item_iterator_t(self).__iter__()
        """
        return func_item_iterator_t(self).__iter__()

    __swig_destroy__ = _ida_funcs.delete_func_t

# Register func_t in _ida_funcs:
_ida_funcs.func_t_swigregister(func_t)
FUNC_NORET = _ida_funcs.FUNC_NORET
r"""
Function doesn't return.
"""

FUNC_FAR = _ida_funcs.FUNC_FAR
r"""
Far function.
"""

FUNC_LIB = _ida_funcs.FUNC_LIB
r"""
Library function.
"""

FUNC_STATICDEF = _ida_funcs.FUNC_STATICDEF
r"""
Static function.
"""

FUNC_FRAME = _ida_funcs.FUNC_FRAME
r"""
Function uses frame pointer (BP)
"""

FUNC_USERFAR = _ida_funcs.FUNC_USERFAR
r"""
User has specified far-ness of the function
"""

FUNC_HIDDEN = _ida_funcs.FUNC_HIDDEN
r"""
A hidden function chunk.
"""

FUNC_THUNK = _ida_funcs.FUNC_THUNK
r"""
Thunk (jump) function.
"""

FUNC_BOTTOMBP = _ida_funcs.FUNC_BOTTOMBP
r"""
BP points to the bottom of the stack frame.
"""

FUNC_NORET_PENDING = _ida_funcs.FUNC_NORET_PENDING
r"""
Function 'non-return' analysis must be performed. This flag is verified upon
func_does_return()
"""

FUNC_SP_READY = _ida_funcs.FUNC_SP_READY
r"""
SP-analysis has been performed. If this flag is on, the stack change points
should not be not modified anymore. Currently this analysis is performed only
for PC
"""

FUNC_FUZZY_SP = _ida_funcs.FUNC_FUZZY_SP
r"""
Function changes SP in untraceable way, for example: and esp, 0FFFFFFF0h
"""

FUNC_PROLOG_OK = _ida_funcs.FUNC_PROLOG_OK
r"""
Prolog analysis has been performed by last SP-analysis
"""

FUNC_PURGED_OK = _ida_funcs.FUNC_PURGED_OK
r"""
'argsize' field has been validated. If this bit is clear and 'argsize' is 0,
then we do not known the real number of bytes removed from the stack. This bit
is handled by the processor module.
"""

FUNC_TAIL = _ida_funcs.FUNC_TAIL
r"""
This is a function tail. Other bits must be clear (except FUNC_HIDDEN).
"""

FUNC_LUMINA = _ida_funcs.FUNC_LUMINA
r"""
Function info is provided by Lumina.
"""

FUNC_OUTLINE = _ida_funcs.FUNC_OUTLINE
r"""
Outlined code, not a real function.
"""

FUNC_REANALYZE = _ida_funcs.FUNC_REANALYZE
r"""
Function frame changed, request to reanalyze the function after the last insn is
analyzed.
"""



def is_func_entry(*args) -> "bool":
    r"""
    is_func_entry(pfn) -> bool
    Does function describe a function entry chunk?

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_funcs.is_func_entry(*args)

def is_func_tail(*args) -> "bool":
    r"""
    is_func_tail(pfn) -> bool
    Does function describe a function tail chunk?

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_funcs.is_func_tail(*args)

def lock_func_range(*args) -> "void":
    r"""
    lock_func_range(pfn, lock)
    Lock function pointer Locked pointers are guaranteed to remain valid until they
    are unlocked. Ranges with locked pointers cannot be deleted or moved.

    @param pfn: (C++: const func_t *) func_t const *
    @param lock: (C++: bool)
    """
    return _ida_funcs.lock_func_range(*args)
class lock_func(object):
    r"""
    Proxy of C++ lock_func class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self, _pfn) -> lock_func

        @param _pfn: func_t const *
        """
        _ida_funcs.lock_func_swiginit(self, _ida_funcs.new_lock_func(*args))
    __swig_destroy__ = _ida_funcs.delete_lock_func

# Register lock_func in _ida_funcs:
_ida_funcs.lock_func_swigregister(lock_func)


def is_func_locked(*args) -> "bool":
    r"""
    is_func_locked(pfn) -> bool
    Is the function pointer locked?

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_funcs.is_func_locked(*args)

def get_func(*args) -> "func_t *":
    r"""
    get_func(ea) -> func_t
    Get pointer to function structure by address.

    @param ea: (C++: ea_t) any address in a function
    @return: ptr to a function or nullptr. This function returns a function entry
             chunk.
    """
    return _ida_funcs.get_func(*args)

def get_func_chunknum(*args) -> "int":
    r"""
    get_func_chunknum(pfn, ea) -> int
    Get the containing tail chunk of 'ea'.
    @retval -1: means 'does not contain ea'
    @retval 0: means the 'pfn' itself contains ea
    @retval >0: the number of the containing function tail chunk

    @param pfn: (C++: func_t *)
    @param ea: (C++: ea_t)
    """
    return _ida_funcs.get_func_chunknum(*args)

def func_contains(*args) -> "bool":
    r"""
    func_contains(pfn, ea) -> bool
    Does the given function contain the given address?

    @param pfn: (C++: func_t *)
    @param ea: (C++: ea_t)
    """
    return _ida_funcs.func_contains(*args)

def is_same_func(*args) -> "bool":
    r"""
    is_same_func(ea1, ea2) -> bool
    Do two addresses belong to the same function?

    @param ea1: (C++: ea_t)
    @param ea2: (C++: ea_t)
    """
    return _ida_funcs.is_same_func(*args)

def getn_func(*args) -> "func_t *":
    r"""
    getn_func(n) -> func_t
    Get pointer to function structure by number.

    @param n: (C++: size_t) number of function, is in range 0..get_func_qty()-1
    @return: ptr to a function or nullptr. This function returns a function entry
             chunk.
    """
    return _ida_funcs.getn_func(*args)

def get_func_qty(*args) -> "size_t":
    r"""
    get_func_qty() -> size_t
    Get total number of functions in the program.
    """
    return _ida_funcs.get_func_qty(*args)

def get_func_num(*args) -> "int":
    r"""
    get_func_num(ea) -> int
    Get ordinal number of a function.

    @param ea: (C++: ea_t) any address in the function
    @return: number of function (0..get_func_qty()-1). -1 means 'no function at the
             specified address'.
    """
    return _ida_funcs.get_func_num(*args)

def get_prev_func(*args) -> "func_t *":
    r"""
    get_prev_func(ea) -> func_t
    Get pointer to the previous function.

    @param ea: (C++: ea_t) any address in the program
    @return: ptr to function or nullptr if previous function doesn't exist
    """
    return _ida_funcs.get_prev_func(*args)

def get_next_func(*args) -> "func_t *":
    r"""
    get_next_func(ea) -> func_t
    Get pointer to the next function.

    @param ea: (C++: ea_t) any address in the program
    @return: ptr to function or nullptr if next function doesn't exist
    """
    return _ida_funcs.get_next_func(*args)

def get_func_ranges(*args) -> "ea_t":
    r"""
    get_func_ranges(ranges, pfn) -> ea_t
    Get function ranges.

    @param ranges: (C++: rangeset_t *) buffer to receive the range info
    @param pfn: (C++: func_t *) ptr to function structure
    @return: end address of the last function range (BADADDR-error)
    """
    return _ida_funcs.get_func_ranges(*args)

def get_func_cmt(*args) -> "qstring *":
    r"""
    get_func_cmt(pfn, repeatable) -> str
    Get function comment.

    @param pfn: (C++: const func_t *) ptr to function structure
    @param repeatable: (C++: bool) get repeatable comment?
    @return: size of comment or -1 In fact this function works with function chunks
             too.
    """
    return _ida_funcs.get_func_cmt(*args)

def set_func_cmt(*args) -> "bool":
    r"""
    set_func_cmt(pfn, cmt, repeatable) -> bool
    Set function comment. This function works with function chunks too.

    @param pfn: (C++: const func_t *) ptr to function structure
    @param cmt: (C++: const char *) comment string, may be multiline (with '
    '). Use empty str ("") to delete comment
    @param repeatable: (C++: bool) set repeatable comment?
    """
    return _ida_funcs.set_func_cmt(*args)

def update_func(*args) -> "bool":
    r"""
    update_func(pfn) -> bool
    Update information about a function in the database (func_t). You must not
    change the function start and end addresses using this function. Use
    set_func_start() and set_func_end() for it.

    @param pfn: (C++: func_t *) ptr to function structure
    @return: success
    """
    return _ida_funcs.update_func(*args)

def add_func_ex(*args) -> "bool":
    r"""
    add_func_ex(pfn) -> bool
    Add a new function. If the fn->end_ea is BADADDR, then IDA will try to determine
    the function bounds by calling find_func_bounds(..., FIND_FUNC_DEFINE).

    @param pfn: (C++: func_t *) ptr to filled function structure
    @return: success
    """
    return _ida_funcs.add_func_ex(*args)

def add_func(*args) -> "bool":
    r"""
    add_func(ea1, ea2=BADADDR) -> bool
    Add a new function. If the function end address is BADADDR, then IDA will try to
    determine the function bounds by calling find_func_bounds(...,
    FIND_FUNC_DEFINE).

    @param ea1: (C++: ea_t) start address
    @param ea2: (C++: ea_t) end address
    @return: success
    """
    return _ida_funcs.add_func(*args)

def del_func(*args) -> "bool":
    r"""
    del_func(ea) -> bool
    Delete a function.

    @param ea: (C++: ea_t) any address in the function entry chunk
    @return: success
    """
    return _ida_funcs.del_func(*args)

def set_func_start(*args) -> "int":
    r"""
    set_func_start(ea, newstart) -> int
    Move function chunk start address.

    @param ea: (C++: ea_t) any address in the function
    @param newstart: (C++: ea_t) new end address of the function
    @return: Function move result codes
    """
    return _ida_funcs.set_func_start(*args)
MOVE_FUNC_OK = _ida_funcs.MOVE_FUNC_OK
r"""
ok
"""

MOVE_FUNC_NOCODE = _ida_funcs.MOVE_FUNC_NOCODE
r"""
no instruction at 'newstart'
"""

MOVE_FUNC_BADSTART = _ida_funcs.MOVE_FUNC_BADSTART
r"""
bad new start address
"""

MOVE_FUNC_NOFUNC = _ida_funcs.MOVE_FUNC_NOFUNC
r"""
no function at 'ea'
"""

MOVE_FUNC_REFUSED = _ida_funcs.MOVE_FUNC_REFUSED
r"""
a plugin refused the action
"""


def set_func_end(*args) -> "bool":
    r"""
    set_func_end(ea, newend) -> bool
    Move function chunk end address.

    @param ea: (C++: ea_t) any address in the function
    @param newend: (C++: ea_t) new end address of the function
    @return: success
    """
    return _ida_funcs.set_func_end(*args)

def reanalyze_function(*args) -> "void":
    r"""
    reanalyze_function(pfn, ea1=0, ea2=BADADDR, analyze_parents=False)
    Reanalyze a function. This function plans to analyzes all chunks of the given
    function. Optional parameters (ea1, ea2) may be used to narrow the analyzed
    range.

    @param pfn: (C++: func_t *) pointer to a function
    @param ea1: (C++: ea_t) start of the range to analyze
    @param ea2: (C++: ea_t) end of range to analyze
    @param analyze_parents: (C++: bool) meaningful only if pfn points to a function tail. if
                            true, all tail parents will be reanalyzed. if false,
                            only the given tail will be reanalyzed.
    """
    return _ida_funcs.reanalyze_function(*args)

def find_func_bounds(*args) -> "int":
    r"""
    find_func_bounds(nfn, flags) -> int
    Determine the boundaries of a new function. This function tries to find the
    start and end addresses of a new function. It calls the module with
    processor_t::func_bounds in order to fine tune the function boundaries.

    @param nfn: (C++: func_t *) structure to fill with information \ nfn->start_ea points to the
                start address of the new function.
    @param flags: (C++: int) Find function bounds flags
    @return: Find function bounds result codes
    """
    return _ida_funcs.find_func_bounds(*args)
FIND_FUNC_NORMAL = _ida_funcs.FIND_FUNC_NORMAL
r"""
stop processing if undefined byte is encountered
"""

FIND_FUNC_DEFINE = _ida_funcs.FIND_FUNC_DEFINE
r"""
create instruction if undefined byte is encountered
"""

FIND_FUNC_IGNOREFN = _ida_funcs.FIND_FUNC_IGNOREFN
r"""
ignore existing function boundaries. by default the function returns function
boundaries if ea belongs to a function.
"""

FIND_FUNC_KEEPBD = _ida_funcs.FIND_FUNC_KEEPBD
r"""
do not modify incoming function boundaries, just create instructions inside the
boundaries.
"""

FIND_FUNC_UNDEF = _ida_funcs.FIND_FUNC_UNDEF
r"""
function has instructions that pass execution flow to unexplored bytes.
nfn->end_ea will have the address of the unexplored byte.
"""

FIND_FUNC_OK = _ida_funcs.FIND_FUNC_OK
r"""
ok, 'nfn' is ready for add_func()
"""

FIND_FUNC_EXIST = _ida_funcs.FIND_FUNC_EXIST
r"""
function exists already. its bounds are returned in 'nfn'.
"""


def get_func_name(*args) -> "qstring *":
    r"""
    get_func_name(ea) -> str
    Get function name.

    @param ea: (C++: ea_t) any address in the function
    @return: length of the function name
    """
    return _ida_funcs.get_func_name(*args)

def calc_func_size(*args) -> "asize_t":
    r"""
    calc_func_size(pfn) -> asize_t
    Calculate function size. This function takes into account all fragments of the
    function.

    @param pfn: (C++: func_t *) ptr to function structure
    """
    return _ida_funcs.calc_func_size(*args)

def get_func_bitness(*args) -> "int":
    r"""
    get_func_bitness(pfn) -> int
    Get function bitness (which is equal to the function segment bitness).
    pfn==nullptr => returns 0
    @retval 0: 16
    @retval 1: 32
    @retval 2: 64

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_funcs.get_func_bitness(*args)

def get_func_bits(*args) -> "int":
    r"""
    get_func_bits(pfn) -> int
    Get number of bits in the function addressing.

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_funcs.get_func_bits(*args)

def get_func_bytes(*args) -> "int":
    r"""
    get_func_bytes(pfn) -> int
    Get number of bytes in the function addressing.

    @param pfn: (C++: const func_t *) func_t const *
    """
    return _ida_funcs.get_func_bytes(*args)

def is_visible_func(*args) -> "bool":
    r"""
    is_visible_func(pfn) -> bool
    Is the function visible (not hidden)?

    @param pfn: (C++: func_t *)
    """
    return _ida_funcs.is_visible_func(*args)

def is_finally_visible_func(*args) -> "bool":
    r"""
    is_finally_visible_func(pfn) -> bool
    Is the function visible (event after considering SCF_SHHID_FUNC)?

    @param pfn: (C++: func_t *)
    """
    return _ida_funcs.is_finally_visible_func(*args)

def set_visible_func(*args) -> "void":
    r"""
    set_visible_func(pfn, visible)
    Set visibility of function.

    @param pfn: (C++: func_t *)
    @param visible: (C++: bool)
    """
    return _ida_funcs.set_visible_func(*args)

def set_func_name_if_jumpfunc(*args) -> "int":
    r"""
    set_func_name_if_jumpfunc(pfn, oldname) -> int
    Give a meaningful name to function if it consists of only 'jump' instruction.

    @param pfn: (C++: func_t *) pointer to function (may be nullptr)
    @param oldname: (C++: const char *) old name of function. if old name was in "j_..." form, then we
                    may discard it and set a new name. if oldname is not known, you
                    may pass nullptr.
    @return: success
    """
    return _ida_funcs.set_func_name_if_jumpfunc(*args)

def calc_thunk_func_target(*args) -> "ea_t *":
    r"""
    calc_thunk_func_target(pfn) -> ea_t
    Calculate target of a thunk function.

    @param pfn: (C++: func_t *) pointer to function (may not be nullptr)
    @return: the target function or BADADDR
    """
    return _ida_funcs.calc_thunk_func_target(*args)

def func_does_return(*args) -> "bool":
    r"""
    func_does_return(callee) -> bool
    Does the function return?. To calculate the answer, FUNC_NORET flag and
    is_noret() are consulted The latter is required for imported functions in the
    .idata section. Since in .idata we have only function pointers but not
    functions, we have to introduce a special flag for them.

    @param callee: (C++: ea_t)
    """
    return _ida_funcs.func_does_return(*args)

def reanalyze_noret_flag(*args) -> "bool":
    r"""
    reanalyze_noret_flag(ea) -> bool
    Plan to reanalyze noret flag. This function does not remove FUNC_NORET if it is
    already present. It just plans to reanalysis.

    @param ea: (C++: ea_t)
    """
    return _ida_funcs.reanalyze_noret_flag(*args)

def set_noret_insn(*args) -> "bool":
    r"""
    set_noret_insn(insn_ea, noret) -> bool
    Signal a non-returning instruction. This function can be used by the processor
    module to tell the kernel about non-returning instructions (like call exit). The
    kernel will perform the global function analysis and find out if the function
    returns at all. This analysis will be done at the first call to
    func_does_return()

    @param insn_ea: (C++: ea_t)
    @param noret: (C++: bool)
    @return: true if the instruction 'noret' flag has been changed
    """
    return _ida_funcs.set_noret_insn(*args)

def get_fchunk(*args) -> "func_t *":
    r"""
    get_fchunk(ea) -> func_t
    Get pointer to function chunk structure by address.

    @param ea: (C++: ea_t) any address in a function chunk
    @return: ptr to a function chunk or nullptr. This function may return a function
             entry as well as a function tail.
    """
    return _ida_funcs.get_fchunk(*args)

def getn_fchunk(*args) -> "func_t *":
    r"""
    getn_fchunk(n) -> func_t
    Get pointer to function chunk structure by number.

    @param n: (C++: int) number of function chunk, is in range 0..get_fchunk_qty()-1
    @return: ptr to a function chunk or nullptr. This function may return a function
             entry as well as a function tail.
    """
    return _ida_funcs.getn_fchunk(*args)

def get_fchunk_qty(*args) -> "size_t":
    r"""
    get_fchunk_qty() -> size_t
    Get total number of function chunks in the program.
    """
    return _ida_funcs.get_fchunk_qty(*args)

def get_fchunk_num(*args) -> "int":
    r"""
    get_fchunk_num(ea) -> int
    Get ordinal number of a function chunk in the global list of function chunks.

    @param ea: (C++: ea_t) any address in the function chunk
    @return: number of function chunk (0..get_fchunk_qty()-1). -1 means 'no function
             chunk at the specified address'.
    """
    return _ida_funcs.get_fchunk_num(*args)

def get_prev_fchunk(*args) -> "func_t *":
    r"""
    get_prev_fchunk(ea) -> func_t
    Get pointer to the previous function chunk in the global list.

    @param ea: (C++: ea_t) any address in the program
    @return: ptr to function chunk or nullptr if previous function chunk doesn't
             exist
    """
    return _ida_funcs.get_prev_fchunk(*args)

def get_next_fchunk(*args) -> "func_t *":
    r"""
    get_next_fchunk(ea) -> func_t
    Get pointer to the next function chunk in the global list.

    @param ea: (C++: ea_t) any address in the program
    @return: ptr to function chunk or nullptr if next function chunk doesn't exist
    """
    return _ida_funcs.get_next_fchunk(*args)

def append_func_tail(*args) -> "bool":
    r"""
    append_func_tail(pfn, ea1, ea2) -> bool
    Append a new tail chunk to the function definition. If the tail already exists,
    then it will simply be added to the function tail list Otherwise a new tail will
    be created and its owner will be set to be our function If a new tail cannot be
    created, then this function will fail.

    @param pfn: (C++: func_t *) pointer to the function
    @param ea1: (C++: ea_t) start of the tail. If a tail already exists at the specified address
                it must start at 'ea1'
    @param ea2: (C++: ea_t) end of the tail. If a tail already exists at the specified address
                it must end at 'ea2'. If specified as BADADDR, IDA will determine
                the end address itself.
    """
    return _ida_funcs.append_func_tail(*args)

def remove_func_tail(*args) -> "bool":
    r"""
    remove_func_tail(pfn, tail_ea) -> bool
    Remove a function tail. If the tail belongs only to one function, it will be
    completely removed. Otherwise if the function was the tail owner, the first
    function using this tail becomes the owner of the tail.

    @param pfn: (C++: func_t *) pointer to the function
    @param tail_ea: (C++: ea_t) any address inside the tail to remove
    """
    return _ida_funcs.remove_func_tail(*args)

def set_tail_owner(*args) -> "bool":
    r"""
    set_tail_owner(fnt, new_owner) -> bool
    Set a new owner of a function tail. The new owner function must be already
    referring to the tail (after append_func_tail).

    @param fnt: (C++: func_t *) pointer to the function tail
    @param new_owner: (C++: ea_t) the entry point of the new owner function
    """
    return _ida_funcs.set_tail_owner(*args)

def func_tail_iterator_set(*args) -> "bool":
    r"""
    func_tail_iterator_set(fti, pfn, ea) -> bool

    @param fti: func_tail_iterator_t *
    @param pfn: func_t *
    @param ea: ea_t
    """
    return _ida_funcs.func_tail_iterator_set(*args)

def func_tail_iterator_set_ea(*args) -> "bool":
    r"""
    func_tail_iterator_set_ea(fti, ea) -> bool

    @param fti: func_tail_iterator_t *
    @param ea: ea_t
    """
    return _ida_funcs.func_tail_iterator_set_ea(*args)

def func_parent_iterator_set(*args) -> "bool":
    r"""
    func_parent_iterator_set(fpi, pfn) -> bool

    @param fpi: func_parent_iterator_t *
    @param pfn: func_t *
    """
    return _ida_funcs.func_parent_iterator_set(*args)

def f_any(*args) -> "bool":
    r"""
    f_any(arg1, arg2) -> bool
    Helper function to accept any address.

    @param arg1: flags64_t
    @param arg2: void *
    """
    return _ida_funcs.f_any(*args)
class func_tail_iterator_t(object):
    r"""
    Proxy of C++ func_tail_iterator_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> func_tail_iterator_t
        __init__(self, _pfn, ea=BADADDR) -> func_tail_iterator_t

        @param _pfn: func_t *
        @param ea: ea_t
        """
        _ida_funcs.func_tail_iterator_t_swiginit(self, _ida_funcs.new_func_tail_iterator_t(*args))
    __swig_destroy__ = _ida_funcs.delete_func_tail_iterator_t

    def set(self, *args) -> "bool":
        r"""
        set(self, _pfn, ea=BADADDR) -> bool

        @param _pfn: func_t *
        @param ea: ea_t
        """
        return _ida_funcs.func_tail_iterator_t_set(self, *args)

    def set_ea(self, *args) -> "bool":
        r"""
        set_ea(self, ea) -> bool

        @param ea: ea_t
        """
        return _ida_funcs.func_tail_iterator_t_set_ea(self, *args)

    def set_range(self, *args) -> "bool":
        r"""
        set_range(self, ea1, ea2) -> bool

        @param ea1: ea_t
        @param ea2: ea_t
        """
        return _ida_funcs.func_tail_iterator_t_set_range(self, *args)

    def chunk(self, *args) -> "range_t const &":
        r"""
        chunk(self) -> range_t
        """
        return _ida_funcs.func_tail_iterator_t_chunk(self, *args)

    def first(self, *args) -> "bool":
        r"""
        first(self) -> bool
        """
        return _ida_funcs.func_tail_iterator_t_first(self, *args)

    def last(self, *args) -> "bool":
        r"""
        last(self) -> bool
        """
        return _ida_funcs.func_tail_iterator_t_last(self, *args)

    def __next__(self, *args) -> "bool":
        r"""
        __next__(self) -> bool
        """
        return _ida_funcs.func_tail_iterator_t___next__(self, *args)

    def prev(self, *args) -> "bool":
        r"""
        prev(self) -> bool
        """
        return _ida_funcs.func_tail_iterator_t_prev(self, *args)

    def main(self, *args) -> "bool":
        r"""
        main(self) -> bool
        """
        return _ida_funcs.func_tail_iterator_t_main(self, *args)

    def __iter__(self):
        r"""
        Provide an iterator on function tails
        """
        ok = self.main()
        while ok:
            yield self.chunk()
            ok = self.next()


    next = __next__


# Register func_tail_iterator_t in _ida_funcs:
_ida_funcs.func_tail_iterator_t_swigregister(func_tail_iterator_t)

class func_item_iterator_t(object):
    r"""
    Proxy of C++ func_item_iterator_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> func_item_iterator_t
        __init__(self, pfn, _ea=BADADDR) -> func_item_iterator_t

        @param pfn: func_t *
        @param _ea: ea_t
        """
        _ida_funcs.func_item_iterator_t_swiginit(self, _ida_funcs.new_func_item_iterator_t(*args))

    def set(self, *args) -> "bool":
        r"""
        set(self, pfn, _ea=BADADDR) -> bool
        Set a function range. if pfn == nullptr then a segment range will be set.

        @param pfn: (C++: func_t *)
        @param _ea: (C++: ea_t)
        """
        return _ida_funcs.func_item_iterator_t_set(self, *args)

    def set_range(self, *args) -> "bool":
        r"""
        set_range(self, ea1, ea2) -> bool
        Set an arbitrary range.

        @param ea1: (C++: ea_t)
        @param ea2: (C++: ea_t)
        """
        return _ida_funcs.func_item_iterator_t_set_range(self, *args)

    def first(self, *args) -> "bool":
        r"""
        first(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_first(self, *args)

    def last(self, *args) -> "bool":
        r"""
        last(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_last(self, *args)

    def current(self, *args) -> "ea_t":
        r"""
        current(self) -> ea_t
        """
        return _ida_funcs.func_item_iterator_t_current(self, *args)

    def chunk(self, *args) -> "range_t const &":
        r"""
        chunk(self) -> range_t
        """
        return _ida_funcs.func_item_iterator_t_chunk(self, *args)

    def __next__(self, *args) -> "bool":
        r"""
        __next__(self, func) -> bool

        @param func: testf_t *
        """
        return _ida_funcs.func_item_iterator_t___next__(self, *args)

    def prev(self, *args) -> "bool":
        r"""
        prev(self, func) -> bool

        @param func: testf_t *
        """
        return _ida_funcs.func_item_iterator_t_prev(self, *args)

    def next_addr(self, *args) -> "bool":
        r"""
        next_addr(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_next_addr(self, *args)

    def next_head(self, *args) -> "bool":
        r"""
        next_head(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_next_head(self, *args)

    def next_code(self, *args) -> "bool":
        r"""
        next_code(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_next_code(self, *args)

    def next_data(self, *args) -> "bool":
        r"""
        next_data(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_next_data(self, *args)

    def next_not_tail(self, *args) -> "bool":
        r"""
        next_not_tail(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_next_not_tail(self, *args)

    def prev_addr(self, *args) -> "bool":
        r"""
        prev_addr(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_prev_addr(self, *args)

    def prev_head(self, *args) -> "bool":
        r"""
        prev_head(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_prev_head(self, *args)

    def prev_code(self, *args) -> "bool":
        r"""
        prev_code(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_prev_code(self, *args)

    def prev_data(self, *args) -> "bool":
        r"""
        prev_data(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_prev_data(self, *args)

    def prev_not_tail(self, *args) -> "bool":
        r"""
        prev_not_tail(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_prev_not_tail(self, *args)

    def decode_prev_insn(self, *args) -> "bool":
        r"""
        decode_prev_insn(self, out) -> bool

        @param out: insn_t *
        """
        return _ida_funcs.func_item_iterator_t_decode_prev_insn(self, *args)

    def decode_preceding_insn(self, *args) -> "bool":
        r"""
        decode_preceding_insn(self, visited, p_farref, out) -> bool

        @param visited: eavec_t *
        @param p_farref: bool *
        @param out: insn_t *
        """
        return _ida_funcs.func_item_iterator_t_decode_preceding_insn(self, *args)

    def succ(self, *args) -> "bool":
        r"""
        succ(self, func) -> bool
        Similar to next(), but succ() iterates the chunks from low to high addresses,
        while next() iterates through chunks starting at the function entry chunk

        @param func: (C++: testf_t *)
        """
        return _ida_funcs.func_item_iterator_t_succ(self, *args)

    def succ_code(self, *args) -> "bool":
        r"""
        succ_code(self) -> bool
        """
        return _ida_funcs.func_item_iterator_t_succ_code(self, *args)

    def __iter__(self):
        r"""
        Provide an iterator on code items
        """
        ok = self.first()
        while ok:
            yield self.current()
            ok = self.next_code()


    next = __next__


    def addresses(self):
        r"""
        Provide an iterator on addresses contained within the function
        """
        ok = self.first()
        while ok:
            yield self.current()
            ok = self.next_addr()


    def code_items(self):
        r"""
        Provide an iterator on code items contained within the function
        """
        ok = self.first()
        while ok:
            yield self.current()
            ok = self.next_code()


    def data_items(self):
        r"""
        Provide an iterator on data items contained within the function
        """
        ok = self.first()
        while ok:
            yield self.current()
            ok = self.next_data()


    def head_items(self):
        r"""
        Provide an iterator on item heads contained within the function
        """
        ok = self.first()
        while ok:
            yield self.current()
            ok = self.next_head()


    def not_tails(self):
        r"""
        Provide an iterator on non-tail addresses contained within the function
        """
        ok = self.first()
        while ok:
            yield self.current()
            ok = self.next_not_tail()

    __swig_destroy__ = _ida_funcs.delete_func_item_iterator_t

# Register func_item_iterator_t in _ida_funcs:
_ida_funcs.func_item_iterator_t_swigregister(func_item_iterator_t)

class func_parent_iterator_t(object):
    r"""
    Proxy of C++ func_parent_iterator_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> func_parent_iterator_t
        __init__(self, _fnt) -> func_parent_iterator_t

        @param _fnt: func_t *
        """
        _ida_funcs.func_parent_iterator_t_swiginit(self, _ida_funcs.new_func_parent_iterator_t(*args))
    __swig_destroy__ = _ida_funcs.delete_func_parent_iterator_t

    def set(self, *args) -> "bool":
        r"""
        set(self, _fnt) -> bool

        @param _fnt: func_t *
        """
        return _ida_funcs.func_parent_iterator_t_set(self, *args)

    def parent(self, *args) -> "ea_t":
        r"""
        parent(self) -> ea_t
        """
        return _ida_funcs.func_parent_iterator_t_parent(self, *args)

    def first(self, *args) -> "bool":
        r"""
        first(self) -> bool
        """
        return _ida_funcs.func_parent_iterator_t_first(self, *args)

    def last(self, *args) -> "bool":
        r"""
        last(self) -> bool
        """
        return _ida_funcs.func_parent_iterator_t_last(self, *args)

    def __next__(self, *args) -> "bool":
        r"""
        __next__(self) -> bool
        """
        return _ida_funcs.func_parent_iterator_t___next__(self, *args)

    def prev(self, *args) -> "bool":
        r"""
        prev(self) -> bool
        """
        return _ida_funcs.func_parent_iterator_t_prev(self, *args)

    def reset_fnt(self, *args) -> "void":
        r"""
        reset_fnt(self, _fnt)

        @param _fnt: func_t *
        """
        return _ida_funcs.func_parent_iterator_t_reset_fnt(self, *args)

    def __iter__(self):
        r"""
        Provide an iterator on function parents
        """
        ok = self.first()
        while ok:
            yield self.parent()
            ok = self.next()


    next = __next__


# Register func_parent_iterator_t in _ida_funcs:
_ida_funcs.func_parent_iterator_t_swigregister(func_parent_iterator_t)


def get_prev_func_addr(*args) -> "ea_t":
    r"""
    get_prev_func_addr(pfn, ea) -> ea_t

    @param pfn: func_t *
    @param ea: ea_t
    """
    return _ida_funcs.get_prev_func_addr(*args)

def get_next_func_addr(*args) -> "ea_t":
    r"""
    get_next_func_addr(pfn, ea) -> ea_t

    @param pfn: func_t *
    @param ea: ea_t
    """
    return _ida_funcs.get_next_func_addr(*args)

def read_regargs(*args) -> "void":
    r"""
    read_regargs(pfn)

    @param pfn: func_t *
    """
    return _ida_funcs.read_regargs(*args)

def add_regarg(*args) -> "void":
    r"""
    add_regarg(pfn, reg, tif, name)

    @param pfn: func_t *
    @param reg: int
    @param tif: tinfo_t const &
    @param name: char const *
    """
    return _ida_funcs.add_regarg(*args)
IDASGN_OK = _ida_funcs.IDASGN_OK
r"""
ok
"""

IDASGN_BADARG = _ida_funcs.IDASGN_BADARG
r"""
bad number of signature
"""

IDASGN_APPLIED = _ida_funcs.IDASGN_APPLIED
r"""
signature is already applied
"""

IDASGN_CURRENT = _ida_funcs.IDASGN_CURRENT
r"""
signature is currently being applied
"""

IDASGN_PLANNED = _ida_funcs.IDASGN_PLANNED
r"""
signature is planned to be applied
"""


def plan_to_apply_idasgn(*args) -> "int":
    r"""
    plan_to_apply_idasgn(fname) -> int
    Add a signature file to the list of planned signature files.

    @param fname: (C++: const char *) file name. should not contain directory part.
    @return: 0 if failed, otherwise number of planned (and applied) signatures
    """
    return _ida_funcs.plan_to_apply_idasgn(*args)

def apply_idasgn_to(*args) -> "int":
    r"""
    apply_idasgn_to(signame, ea, is_startup) -> int
    Apply a signature file to the specified address.

    @param signame: (C++: const char *) short name of signature file (the file name without path)
    @param ea: (C++: ea_t) address to apply the signature
    @param is_startup: (C++: bool) if set, then the signature is treated as a startup one for
                       startup signature ida doesn't rename the first function of
                       the applied module.
    @return: Library function codes
    """
    return _ida_funcs.apply_idasgn_to(*args)

def get_idasgn_qty(*args) -> "int":
    r"""
    get_idasgn_qty() -> int
    Get number of signatures in the list of planned and applied signatures.

    @return: 0..n
    """
    return _ida_funcs.get_idasgn_qty(*args)

def get_current_idasgn(*args) -> "int":
    r"""
    get_current_idasgn() -> int
    Get number of the the current signature.

    @return: 0..n-1
    """
    return _ida_funcs.get_current_idasgn(*args)

def calc_idasgn_state(*args) -> "int":
    r"""
    calc_idasgn_state(n) -> int
    Get state of a signature in the list of planned signatures

    @param n: (C++: int) number of signature in the list (0..get_idasgn_qty()-1)
    @return: state of signature or IDASGN_BADARG
    """
    return _ida_funcs.calc_idasgn_state(*args)

def del_idasgn(*args) -> "int":
    r"""
    del_idasgn(n) -> int
    Remove signature from the list of planned signatures.

    @param n: (C++: int) number of signature in the list (0..get_idasgn_qty()-1)
    @return: IDASGN_OK, IDASGN_BADARG, IDASGN_APPLIED
    """
    return _ida_funcs.del_idasgn(*args)

def get_idasgn_title(*args) -> "qstring *":
    r"""
    get_idasgn_title(name) -> str
    Get full description of the signature by its short name.

    @param name: (C++: const char *) short name of a signature
    @return: size of signature description or -1
    """
    return _ida_funcs.get_idasgn_title(*args)

def apply_startup_sig(*args) -> "bool":
    r"""
    apply_startup_sig(ea, startup) -> bool
    Apply a startup signature file to the specified address.

    @param ea: (C++: ea_t) address to apply the signature to; usually idainfo::start_ea
    @param startup: (C++: const char *) the name of the signature file without path and extension
    @return: true if successfully applied the signature
    """
    return _ida_funcs.apply_startup_sig(*args)

def try_to_add_libfunc(*args) -> "int":
    r"""
    try_to_add_libfunc(ea) -> int
    Apply the currently loaded signature file to the specified address. If a library
    function is found, then create a function and name it accordingly.

    @param ea: (C++: ea_t) any address in the program
    @return: Library function codes
    """
    return _ida_funcs.try_to_add_libfunc(*args)
LIBFUNC_FOUND = _ida_funcs.LIBFUNC_FOUND
r"""
ok, library function is found
"""

LIBFUNC_NONE = _ida_funcs.LIBFUNC_NONE
r"""
no, this is not a library function
"""

LIBFUNC_DELAY = _ida_funcs.LIBFUNC_DELAY
r"""
no decision because of lack of information
"""


def get_fchunk_referer(*args) -> "ea_t":
    r"""
    get_fchunk_referer(ea, idx) -> ea_t

    @param ea: ea_t
    @param idx: size_t
    """
    return _ida_funcs.get_fchunk_referer(*args)

def get_idasgn_desc(*args) -> "PyObject *":
    r"""
    get_idasgn_desc(n) -> (str, str)
    Get information about a signature in the list.
    It returns: (name of signature, names of optional libraries)

    See also: get_idasgn_desc_with_matches

    @param n: number of signature in the list (0..get_idasgn_qty()-1)
    @return: None on failure or tuple(signame, optlibs)
    """
    return _ida_funcs.get_idasgn_desc(*args)

def get_idasgn_desc_with_matches(*args) -> "PyObject *":
    r"""
    get_idasgn_desc_with_matches(n) -> (str, str, int)
    Get information about a signature in the list.
    It returns: (name of signature, names of optional libraries, number of matches)

    @param n: number of signature in the list (0..get_idasgn_qty()-1)
    @return: None on failure or tuple(signame, optlibs, nmatches)
    """
    return _ida_funcs.get_idasgn_desc_with_matches(*args)

def func_t__from_ptrval__(*args) -> "func_t *":
    r"""
    func_t__from_ptrval__(ptrval) -> func_t

    @param ptrval: size_t
    """
    return _ida_funcs.func_t__from_ptrval__(*args)

#<pycode(py_funcs)>
import ida_idaapi
@ida_idaapi.replfun
def calc_thunk_func_target(*args):
    if len(args) == 2:
        pfn, rawptr = args
        target, fptr = calc_thunk_func_target.__dict__["orig"](pfn)
        import ida_pro
        ida_pro.ea_pointer.frompointer(rawptr).assign(fptr)
        return target
    else:
        return calc_thunk_func_target.__dict__["orig"](*args)
#</pycode(py_funcs)>







#!/usr/bin/env python
# -----------------------------------------------------------------------
# IDAPython - Python plugin for Interactive Disassembler
#
# Copyright (c) The IDAPython Team <idapython@googlegroups.com>
#
# All rights reserved.
#
# For detailed copyright information see the file COPYING in
# the root of the distribution archive.
# -----------------------------------------------------------------------
# init.py - Essential init routines
# -----------------------------------------------------------------------
from __future__ import print_function
import os
import sys
import time
import warnings
import os
import os.path

if os.name == 'nt' and \
    sys.version_info.major == 3 and \
    sys.version_info.minor >= 11:
    # Python 3.11 has a bug with DLLs directory missing from sys.path
    # so add it if it's not there
    base = sys.base_exec_prefix
    dllspath = os.path.join(base, sys.platlibdir)
    if os.path.exists(dllspath) and dllspath not in sys.path:
        i = sys.path.index(base) if base in sys.path else len(sys.path)
        sys.path.insert(i, dllspath)


# Prepare sys.path so loading of the shared objects works
lib_dynload = os.path.join(
    sys.executable,
    IDAPYTHON_DYNLOAD_BASE,
    "python",
    str(sys.version_info.major))

is_x64 = sys.maxsize >= 0x100000000
if is_x64:
    # x64 python requires our lib_dynload to be added; sys.path seems
    # to be composed differently than x86 builds.
    # In addition, we always want our own lib-dynload to come first:
    # the PyQt (& sip) modules that might have to be loaded, should
    # be the ones shipped with IDA and not those possibly available
    # on the system.
    sys.path.insert(0, os.path.join(lib_dynload, IDAPYTHON_DYNLOAD_RELPATH))
    sys.path.insert(0, lib_dynload)
else:
    # for non-x64 platforms, make sure everything works as it used to,
    # by appending our own lib-dynload to sys.argv..
    sys.path.append(os.path.join(lib_dynload, IDAPYTHON_DYNLOAD_RELPATH))

try:
    import ida_idaapi
    import ida_kernwin
    import ida_diskio
except ImportError as e:
    print("Import failed: %s. Current sys.path:" % str(e))
    for p in sys.path:
        print("\t%s" % p)
    raise


# -----------------------------------------------------------------------
# Take over the standard text outputs
# -----------------------------------------------------------------------
class IDAPythonStdOut:
    """
    Dummy file-like class that receives stdout and stderr
    """
    encoding = "UTF-8"

    def write(self, text):
        # NB: in case 'text' is Unicode, msg() will decode it
        # and call msg() to print it
        ida_kernwin.msg(text)

    def flush(self):
        pass

    def isatty(self):
        return False

# -----------------------------------------------------------------------
def runscript(script):
    """
    Executes a script.
    This function is present for backward compatiblity. Please use idaapi.IDAPython_ExecScript() instead

    @param script: script path

    @return: Error string or None on success
    """

    import ida_idaapi
    return ida_idaapi.IDAPython_ExecScript(script, globals())

# -----------------------------------------------------------------------
def print_banner():
    banner = [
      "Python %s " % sys.version,
      "IDAPython" + (" 64-bit" if ida_idaapi.__EA64__ else "") + " v%d.%d.%d %s (serial %d) (c) The IDAPython Team <idapython@googlegroups.com>" % IDAPYTHON_VERSION
    ]
    sepline = '-' * (max([len(s) for s in banner])+1)

    print(sepline)
    print("\n".join(banner))
    print(sepline)

# -----------------------------------------------------------------------

# Redirect stderr and stdout to the IDA message window
_orig_stdout = sys.stdout;
_orig_stderr = sys.stderr;
sys.stdout = sys.stderr = IDAPythonStdOut()

# -----------------------------------------------------------------------
# Initialize the help, with our own stdin wrapper, that'll query the user
# -----------------------------------------------------------------------
import pydoc
class IDAPythonHelpPrompter(object):
    def readline(self):
        return ida_kernwin.ask_str('', 0, 'Help topic?')

class IDAPythonHelp(pydoc.Helper):
    def __init__(self):
        super().__init__(input = IDAPythonHelpPrompter(), output = sys.stdout)
    def help(self, *args):
        try:
            return super().help(*args)
        except ImportError as e:
            print(e)

help = IDAPythonHelp()

# Assign a default sys.argv
sys.argv = [""]

# Have to make sure Python finds our modules
sys.path.append(ida_diskio.idadir("python"))

# Remove current directory from the top of the patch search
if '' in sys.path: # On non Windows, the empty path is added
    sys.path.remove('')

if os.getcwd() in sys.path:
    sys.path.remove(os.getcwd())

# ...and add it to the end if needed
if not IDAPYTHON_REMOVE_CWD_SYS_PATH:
    sys.path.append(os.getcwd())

# Additional $IDAUSR-derived paths
if IDAPYTHON_IDAUSR_SYSPATH:
    idausr_python_list = ida_diskio.get_ida_subdirs("python")
    for idausr_python in idausr_python_list:
        one = os.path.join(idausr_python, str(sys.version_info.major))
        if one not in sys.path:
            sys.path.append(one)

if IDAPYTHON_COMPAT_AUTOIMPORT_MODULES:
    # Import all the required modules
    from idaapi import get_user_idadir, cvar, Appcall, Form
    from idc      import *
    from idautils import *
    import idaapi

# Load the users personal init file
userrc = os.path.join(ida_diskio.get_user_idadir(), "idapythonrc.py")
if os.path.exists(userrc):
    ida_idaapi.IDAPython_ExecScript(userrc, globals())

# In Python3, some modules (e.g., subprocess) will load the 'signal'
# module which, upon loading, will registers default handlers for some
# signals. In particular, for SIGINT, which we don't want to handle
# since it'll prevent us from killing IDA with Ctrl+C on a TTY.
if sys.version_info.major >= 3:
    import signal
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Also, embedded Python3 will not include the 'site packages' by
    # default, which means many packages provided by the distribution
    # would not be reachable. Let's provide a way to load them.
    import site
    for sp in site.getsitepackages():
        if sp not in sys.path:
            sys.path.append(sp)

# All done, ready to rock.



"""
summary: executing code into the application being debugged (on Windows)

description:
  Using the `ida_idd.Appcall` utility to execute code in
  the process being debugged.

  This example will run the test program and stop wherever
  the cursor currently is, and then perform an appcall to
  execute the `ref4` and `ref8` functions.

  To use this example:

    * run `ida64` on test program `simple_appcall_win64.exe`, or
      `ida` on test program `simple_appcall_win32.exe`, and wait for
      auto-analysis to finish
    * select the 'windows debugger' (either local, or remote)
    * run this script

  Note: the real body of code is in `simple_appcall_common.py`.
"""

import os
import sys
sys.path.append(os.path.dirname(__file__))

# Windows binaries don't have any symbols, thus we'll have
# to assign names to addresses of interest before we can
# appcall them by name.
import ida_ida
if ida_ida.inf_is_64bit():
    ref4_ea = 0x140001000
    ref8_ea = 0x140001060
else:
    ref4_ea = 0x401000
    ref8_ea = 0x401050

import simple_appcall_common
appcall_hooks = simple_appcall_common.appcall_hooks_t(
    name_funcs=[
        (ref4_ea, "ref4"),
        (ref8_ea, "ref8"),
    ])

appcall_hooks.hook()
appcall_hooks.run()



"""
summary: using the "Select offsets" widget

description:
  Registers an action opens the "Select offsets" widget
  (select_udt_by_offset() call).

  This effectively repeats the functionality already available
  through Alt+Y.

  Place cursor on the union field and press Shift+T
"""

import ida_idaapi
import ida_hexrays
import ida_lines
import ida_typeinf
import ida_kernwin

# --------------------------------------------------------------------------
class func_stroff_ah_t(ida_kernwin.action_handler_t):
    def __init__(self):
        ida_kernwin.action_handler_t.__init__(self)

    def activate(self, ctx):
        # get the current item
        vu = ida_hexrays.get_widget_vdui(ctx.widget)
        vu.get_current_item(ida_hexrays.USE_KEYBOARD)

        # REGION1, will be referenced later
        # check that the current item is a union field
        if not vu.item.is_citem():
            ida_kernwin.warning("Please position the cursor on a union member")
            return 0
        e = vu.item.e
        while True:
            op = e.op
            if op != ida_hexrays.cot_memptr and op != ida_hexrays.cot_memref:
                ida_kernwin.warning("Please position the cursor on a union member")
                return 0
            e = e.x
            if op == ida_hexrays.cot_memptr:
                if e.type.is_union():
                    break
            else:
                if ida_typeinf.remove_pointer(e.type).is_union():
                    break
            if not e.type.is_udt():
                ida_kernwin.warning("Please position the cursor on a union member")
                return 0
        # END REGION1

        # REGION2
        # calculate the member offset
        off = 0
        e = vu.item.e
        while True:
            e2 = e.x
            tif = ida_typeinf.remove_pointer(e2.type)
            if not tif.is_union():
                off += e.m
            e = e2
            if e2.op != ida_hexrays.cot_memptr and e2.op != ida_hexrays.cot_memref:
                break
            if not e2.type.is_udt():
                break
        # END REGION2

        # REGION3
        # go up and collect more member references (in order to calculate the final offset)
        p = vu.item.e
        while True:
            p2 = vu.cfunc.body.find_parent_of(p)
            if p2.op == ida_hexrays.cot_memptr:
                break
            if p2.op == ida_hexrays.cot_memref:
                e2 = p2.cexpr
                tif = ida_typeinf.remove_pointer(e2.x.type)
                if not tif.is_union():
                    off += e2.m
                p = p2
                continue
            if p2.op == ida_hexrays.cot_ref:
                # handle &a.b + N (this expression may appear if the user previously selected
                #                  a wrong field)
                delta = 0
                add = vu.cfunc.body.find_parent_of(p2)
                if add.op == ida_hexrays.cot_cast:
                    add = vu.cfunc.body.find_parent_of(add)
                if add.op == ida_hexrays.cot_add and add.y.op == ida_hexrays.cot_num:
                    delta = add.y.numval()
                    objsize = add.type.get_ptrarr_objsize()
                    nbytes = delta * objsize
                    off += nbytes
            # we could use helpers like WORD/BYTE/... to calculate a more precise offset
            # if ( p2->op == cot_call && (e2->exflags & EXFL_LVALUE) != 0 )
            break
        # END REGION3

        # REGION4
        ea = vu.item.e.ea
        # the item itself may be unaddressable.
        # TODO: find its addressable parent
        if ea == ida_idaapi.BADADDR:
            ida_kernwin.warning("Sorry, the current item is not addressable")
            return 0
        # END REGION4

        # REGION5
        # prepare the text representation for the item,
        # use the neighborhoods of cursor
        line = ida_lines.tag_remove(ida_kernwin.get_custom_viewer_curline(vu.ct, False))
        line_len = len(line)
        x = max(0, vu.cpos.x - 10)
        l = min(10, line_len - vu.cpos.x) + 10
        line = line[x:x+l]
        # END REGION5

        # REGION6
        ops = ida_hexrays.ui_stroff_ops_t()
        op = ops.push_back()
        op.offset = off
        op.text = line
        # END REGION6

        # REGION7
        class set_union_sel_t(ida_hexrays.ui_stroff_applicator_t):
            def __init__(self, ea):
                ida_hexrays.ui_stroff_applicator_t.__init__(self)
                self.ea = ea

            def apply(self, opnum, path, top_tif, spath):
                typename = ida_typeinf.print_tinfo('', 0, 0, ida_typeinf.PRTYPE_1LINE, top_tif, '', '')
                ida_kernwin.msg("User selected %s of type %s\n" % (spath, typename))
                if path.empty():
                    return False
                vu.cfunc.set_user_union_selection(self.ea, path)
                vu.cfunc.save_user_unions()
                return True
        # END REGION7

        # REGION8
        su = set_union_sel_t(ea)
        res = ida_hexrays.select_udt_by_offset(None, ops, su)
        if res != 0:
            # regenerate ctree
            vu.refresh_view(True)
        # END REGION8

        return 1

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET if \
            ctx.widget_type == ida_kernwin.BWN_PSEUDOCODE else \
            ida_kernwin.AST_DISABLE_FOR_WIDGET


# --------------------------------------------------------------------------
# a plugin interface, boilerplate code
class my_plugin_t(ida_idaapi.plugin_t):
    flags = ida_idaapi.PLUGIN_HIDE
    wanted_name = "Structure offsets (IDAPython)"
    wanted_hotkey = ""
    comment = "Sample plugin17 for Hex-Rays decompiler"
    help = ""
    def init(self):
        if ida_hexrays.init_hexrays_plugin():
            print("Hex-rays version %s has been detected, Structure offsets ready to use" % ida_hexrays.get_hexrays_version())
            ida_kernwin.register_action(
                ida_kernwin.action_desc_t(
                    "vds17:strchoose",
                    "Structure offsets",
                    func_stroff_ah_t(),
                    "Shift+T"))
            return ida_idaapi.PLUGIN_KEEP # keep us in the memory
    def term(self):
        pass
    def run(self, arg):
        pass

def PLUGIN_ENTRY():
    return my_plugin_t()

"""
# A few notes about the VDS17 sample

You can find two VDS17 samples in the IDA Pro install directory:

    python/examples/hexrays/vds17.py
    plugins/hexrays_sdk/plugins/vds17

The former is an IDAPython plugin and the latter is a C++ IDA Pro plugin.
Actually they have the same functionality.
Just to be more concrete the vds17.py plugin will be used below.


## How the user interface works

Let us suppose that we have the following local types:

    1 C     struct {int c0;int c1;}
    2 U     union {int u0;__int16 u1;}
    3 D     struct {int d0;C d1;U d2;}
    4 E     struct {int e0;D e1;}
    5 res_t struct {int r0;int r1;__int16 r2;}

and the decompiler generates the following pseudocode:

    void __cdecl f(res_t *r)
    {
      r->r0 = e->e1.d1.c0;
      r->r1 = e->e1.d2.u0;
      r->r2 = e->e1.d2.u1;
    }

As we see, it looks good, the decompiler did a really good job.

But let us imagine that we need reference to the ```e1.d2.u0``` union member on the last line.

At first we need to load the VDS17 plugin. For that,
place the cursor at the ```u1``` union member on the last line
and use ```Shift-T```.

The "Structure offsets" dialog appears on the screen.
The left pane of the dialog contains the available local types
and the right pane has the following view:

    [checked] e->e1.d2.u1; | 10h | |

Use the left pane to select ```E```, expand it, and select (Double-click) on the ```u0``` member.

The right pane will change to:

    [checked] e->e1.d2.u1; | 10h | [checke] E.e1.d2.u0 |

Since this is what we want, press the ```OK``` button and the pseudocode gets changed:

    void __cdecl f(res_t *r)
    {
      r->r0 = e->e1.d1.c0;
      r->r1 = e->e1.d2.u0;
      r->r2 = e->e1.d2.u0;
    }


### What do we need this plugin for?

All union members have the same offset in the parent structure.
The decompiler selects the first suitable union member.
Sometimes we need to change this selection and use another union member.


## API details

Let us look into ```python/examples/hexrays/vds17.py```.

* REGION1: check that the cursor points to the union member
* REGION2: calculate the member offset
* REGION3: sometimes there is a need to adjust the offset
* REGION4: the pointed item must be addressable

Then the magic begins:

* REGION5: we need something to show in the ```Operand``` column of the right pane.
    The text around the cursor looks like a good compromise.
* REGION6: we are ready to fill the right pane: we have calculated the offset and
    have prepared the descriptive text for the operand.
* REGION7: it is a callback that informs us about the user selection (will be described later)
* REGION8: activate the "Structure offsets" dialog, let the user select,
    the callback specified above will update the union member, refresh pseudocode


### The devil is in the details

The main part of callback is the ```apply``` method (REGION7).
It receives two arguments:

1. ```opnum``` is the number of the selected operand (line) in the right pane of the dialog.
     We need something to map the line number to our operand.
     In the case of VDS17 ```ea``` performs this role.
2. ```path```  the path that describes the union selection.
3. ```top_tif``` typeinfo of the top-level UDT which user selected
4. ```spath``` the field names path to the selected member

The union selection path is denotes a concrete member inside a UDT.

For structure types there is no need in the union selection path because the member
offset uniquely denotes the desired member.

For unions, on the other hand, the member offset is not enough because all union
members start at the same offset zero. For them, we remember the ordinal number of the
union member in the path. E.g., for the local types given above, the following holds:

* e1.d1.c0 is denoted by an empty path because it does not have any unions
* e1.d2.u0 is denoted by path consisting of 0: we use the first member of U
* e1.d2.u1 is denoted by path consisting of 1: we use the second member of U

You can retrieve the union selection path using API call ```get_user_union_selection```
or apply it using ```set_user_union_selection```.
"""



#############################################################################
##
## Copyright (c) 2021 Riverbank Computing Limited <info@riverbankcomputing.com>
## 
## This file is part of PyQt5.
## 
## This software is licensed for use under the terms of the Riverbank Commercial
## License.  See the file LICENSE for more details.  It is supplied WITHOUT ANY
## WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
## A PARTICULAR PURPOSE.
##
#############################################################################


from ..Compiler.proxy_metaclass import ProxyMetaclass


class ProxyBase(object):
    """ A base class for proxies using Python v2 syntax for setting the
    meta-class.
    """

    __metaclass__ = ProxyMetaclass




import idaapi

class Data_t:
    def children(self, path):
        num_children = self._simple_hash(path) & 0x7
        num_folders  = num_children >> 1

        for n in range(num_folders):
            yield True, "Folder %d" % (n + 1)
        for n in range(num_children - num_folders):
            yield False, "Item %d" % (n + 1)

    def _simple_hash(self, path):
        s = 0
        for c in path:
            s += ord(c)
        return s

#-----------------------

# dirspec for the dirtree
class my_dirspec_t(idaapi.dirspec_t):
    def __init__(self):
        idaapi.dirspec_t.__init__(self)
        self.inodes = []
        self.name_index = {}

    def add_entry(self, dirpath, name):
        new_inode = len(self.inodes)
        self.inodes.append(name)
        self.name_index[dirpath, name] = new_inode
        return new_inode

    def get_name(self, inode, flags=0):
        if inode >= 0 and inode < len(self.inodes):
            return self.inodes[inode]

    def get_inode(self, dirpath, name):
        return self.name_index.get((dirpath, name), idaapi.direntry_t.BADIDX)

    def n_inodes(self):
        return len(self.inodes)

    def get_attrs(self, inode):
        pass

    def rename_inode(self, inode, newname):
        pass

    def unlink_inode(self, inode):
        pass

# chooser with dirtree, lazy-loaded
class my_tree_t(idaapi.Choose):
    inherited = idaapi.Choose

    def __init__(self):
        self.dirspec = my_dirspec_t()
        self.dirtree = idaapi.dirtree_t(self.dirspec)

        self.data = Data_t()

        flags = idaapi.CH_TM_FULL_TREE
        columns = [
            ["Name", 10 | idaapi.CHCOL_PLAIN]
        ]

        self.inherited.__init__(self, "My tree", columns, flags)

    def OnGetSize(self):
        return self.dirspec.n_inodes()

    def OnGetLine(self, index):
        inode = self.OnIndexToInode(index)
        return [self.dirspec.get_name(inode)]

    def OnGetDirTree(self):
        return self.dirspec, self.dirtree

    def OnIndexToInode(self, n):
        return n

    # TODO: add "dirtree" parameter when the pywraps are ready
    def OnLazyLoadDir(self, dir_path):
        dirtree = self.dirtree  # should use the parameter when ready

        for is_dir, entry in self.data.children(dir_path):
            if is_dir:
                dirtree.mkdir(entry)
            else:
                inode = self.dirspec.add_entry(dir_path, entry)
                dirtree.link(inode)
        return True

# plugin
class lazy_chooser_plugin_t(idaapi.plugin_t):
    flags = 0
    comment = "This is a comment."
    help = "This is a test."
    wanted_name = "Test a lazy-loader chooser in Python"
    wanted_hotkey = "Alt-Shift-F12"

    def init(self):
        self.tree = my_tree_t()
        print("Registered plugin test-lazy-chooser");
        return idaapi.PLUGIN_KEEP

    def run(self, arg):
        self.tree.Show()

    def term(self):
        print("Un-registered plugin test-lazy-chooser");

def PLUGIN_ENTRY():
    return lazy_chooser_plugin_t()




"""
summary: an easy-to-use way to colorize lines

description:
  This builds upon the `ida_kernwin.UI_Hooks.get_lines_rendering_info`
  feature, to provide a quick & easy way to colorize disassembly
  lines.

  Contrary to @colorize_disassembly, the coloring is not persisted in
  the database, and will therefore be lost after the session.

  By triggering the action multiple times, the user can "carousel"
  across 4 predefined colors (and return to the "no color" state.)

keywords: coloring

see_also: colorize_disassembly
"""

import ida_kernwin
import ida_moves

class on_the_fly_coloring_hooks_t(ida_kernwin.UI_Hooks):

    # We'll offer the users the ability to carousel around the
    # following colors. Well, note that these are in fact not
    # colors, but rather color "keys": each theme might have its
    # own values for those.
    AVAILABLE_COLORS = [
        ida_kernwin.CK_EXTRA5,
        ida_kernwin.CK_EXTRA6,
        ida_kernwin.CK_EXTRA7,
        ida_kernwin.CK_EXTRA8,
    ]

    def __init__(self):
        ida_kernwin.UI_Hooks.__init__(self)

        # Each view can have on-the-fly coloring.
        # We'll store the custom colors keyed on the widget's title
        self.by_widget = {}

    def get_lines_rendering_info(self, out, widget, rin):
        """
        Called by IDA, at rendering-time.

        We'll look in our set of marked lines, and for those that are
        found, will produce additional rendering information for IDA
        to use.
        """
        title = ida_kernwin.get_widget_title(widget)
        assigned = self.by_widget.get(title, None)
        if assigned is not None:
            for section_lines in rin.sections_lines:
                for line in section_lines:
                    for loc, color in assigned:
                        if self._same_lines(widget, line.at, loc.place()):
                            e = ida_kernwin.line_rendering_output_entry_t(line)
                            e.bg_color = color
                            out.entries.push_back(e)

    def _same_lines(self, viewer, p0, p1):
        return ida_kernwin.get_custom_viewer_place_xcoord(viewer, p0, p1) != -1

    def _find_loc_index(self, viewer, assigned, loc):
        for idx, tpl in enumerate(assigned):
            _loc = tpl[0]
            if self._same_lines(viewer, loc.place(), _loc.place()):
                return idx
        return -1

    def carousel_color(self, viewer, title):
        """
        This performs the work of iterating across the available
        colors (and the 'no-color' state.)
        """
        loc = ida_moves.lochist_entry_t()
        if ida_kernwin.get_custom_viewer_location(loc, viewer):
            assigned = self.by_widget.get(title, [])
            new_color = None

            idx = self._find_loc_index(viewer, assigned, loc)
            if idx > -1:
                prev_color = assigned[idx][1]
                prev_color_idx = self.AVAILABLE_COLORS.index(prev_color)
                new_color = None \
                            if prev_color_idx >= (len(self.AVAILABLE_COLORS) - 1) \
                            else self.AVAILABLE_COLORS[prev_color_idx + 1]
            else:
                new_color = self.AVAILABLE_COLORS[0]

            if idx > -1:
                del assigned[idx]
            if new_color is not None:
                assigned.append((loc, new_color))

            if assigned:
                self.by_widget[title] = assigned
            else:
                if title in self.by_widget:
                    del self.by_widget[title]


class carousel_color_ah_t(ida_kernwin.action_handler_t):
    """
    The action that will be invoked by IDA when the user
    activates its shortcut.
    """
    def __init__(self, hooks):
        ida_kernwin.action_handler_t.__init__(self)
        self.hooks = hooks

    def activate(self, ctx):
        v = ida_kernwin.get_current_viewer()
        if v:
            self.hooks.carousel_color(v, ctx.widget_title)
            return 1 # will cause the widget to redraw

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ida_kernwin.get_current_viewer() \
            else ida_kernwin.AST_DISABLE_FOR_WIDGET


ACTION_NAME = "example:colorize_disassembly_on_the_fly"
ACTION_LABEL = "Pick line color"
ACTION_SHORTCUT = "!"
ACTION_HELP = "Press %s to carousel around available colors (or remove a previously-set color)" % ACTION_SHORTCUT

otf_coloring = on_the_fly_coloring_hooks_t()
if ida_kernwin.register_action(ida_kernwin.action_desc_t(
        ACTION_NAME,
        ACTION_LABEL,
        carousel_color_ah_t(otf_coloring),
        ACTION_SHORTCUT)):
    print("Registered action \"%s\". %s" % (ACTION_LABEL, ACTION_HELP))
    otf_coloring.hook()



"""
summary: A widget showing data in a tabular fashion

description:
  Shows how to subclass the ida_kernwin.Choose class to
  show data organized in a simple table.
  In addition, registers a couple actions that can be applied to it.

keywords: chooser, actions

see_also: choose_multi, chooser_with_folders
"""

import ida_kernwin
from ida_kernwin import Choose

# -----------------------------------------------------------------------
class chooser_handler_t(ida_kernwin.action_handler_t):
    def __init__(self, thing):
        ida_kernwin.action_handler_t.__init__(self)
        self.thing = thing

    def activate(self, ctx):
        sel = []
        for idx in ctx.chooser_selection:
            sel.append(str(idx))
        print("command %s selected @ %s" % (self.thing, ", ".join(sel)))

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ida_kernwin.is_chooser_widget(ctx.widget_type) \
          else ida_kernwin.AST_DISABLE_FOR_WIDGET

    @staticmethod
    def compose_action_name(v):
        return "choose:act%s" % v


# create actions
actions_variants = ["A", "B"]
for av in actions_variants:
    actname = chooser_handler_t.compose_action_name(av)
    if ida_kernwin.unregister_action(actname):
        print("Unregistered previously-registered action \"%s\"" % actname)

    desc = ida_kernwin.action_desc_t(actname, "command %s" % av, chooser_handler_t(av))
    if ida_kernwin.register_action(desc):
        print("Registered action \"%s\"" % actname)


# -----------------------------------------------------------------------
class MyChoose(Choose):

    def __init__(self, title, nb = 5, flags = 0,
                 modal = False,
                 embedded = False, width = None, height = None):
        Choose.__init__(
            self,
            title,
            [ ["Address", 10], ["Name", 30] ],
            flags = flags | Choose.CH_RESTORE
                          | (Choose.CH_CAN_INS
                           | Choose.CH_CAN_DEL
                           | Choose.CH_CAN_EDIT
                           | Choose.CH_CAN_REFRESH),
            embedded = embedded,
            width = width,
            height = height)
        self.n = 0
        self.items = [ self.make_item() for x in range(nb) ]
        self.icon = 5
        self.selcount = 0
        self.modal = modal
        self.popup_names = ["Inzert", "Del leet", "Ehdeet", "Ree frech"]

        print("created %s" % str(self))

    def OnInit(self):
        print("inited", str(self))
        return True

    def OnGetSize(self):
        n = len(self.items)
        print("getsize -> %d" % n)
        return n

    def OnGetLine(self, n):
        print("getline %d" % n)
        return self.items[n]

    def OnGetIcon(self, n):
        r = self.items[n]
        t = self.icon + r[1].count("*")
        print("geticon", n, t)
        return t

    def OnGetLineAttr(self, n):
        print("getlineattr %d" % n)
        if n == 1:
            return [0xFF0000, 0]

    def OnInsertLine(self, n):
        # we ignore current selection
        n = self.n # position at the just added item
        self.items.append(self.make_item())
        print("insert line")
        return (Choose.ALL_CHANGED, n)

    def OnDeleteLine(self, n):
        print("del %d " % n)
        del self.items[n]
        return [Choose.ALL_CHANGED] + self.adjust_last_item(n)

    def OnEditLine(self, n):
        self.items[n][1] = self.items[n][1] + "*"
        print("editing %d" % n)
        return (Choose.ALL_CHANGED, n)

    def OnRefresh(self, n):
        print("refresh %d" % n)
        return None # call standard refresh

    def OnSelectLine(self, n):
        self.selcount += 1
        warning("[%02d] selectline '%d'" % (self.selcount, n))
        return (Choose.NOTHING_CHANGED, )

    def OnClose(self):
        print("closed", str(self))

    def show(self):
        ok = self.Show(self.modal) >= 0
        if ok:
            # permanently attach actions to this chooser's popup menu
            for av in actions_variants:
                actname = chooser_handler_t.compose_action_name(av)
                ida_kernwin.attach_action_to_popup(self.GetWidget(), None, actname)
        return ok

    def make_item(self):
        r = [str(self.n), "func_%04d" % self.n]
        self.n += 1
        return r


# -----------------------------------------------------------------------
def test_choose(modal = False, nb = 10):
    global c
    c = MyChoose("Choose - sample 1", nb = nb, modal = modal)
    c.show()

# -----------------------------------------------------------------------
def test_choose_embedded():
    global c
    c = MyChoose("Choose - embedded", nb=12, embedded = True, width=123, height=222)
    r = c.Embedded()
    if r == 0:
        try:
            if test_embedded:
                o, sel = _idaapi.choose_get_embedded(c)
                print("o=%s, type(o)=%s" % (str(o), type(o)))
                test_embedded(o)
        finally:
            c.Close()

# -----------------------------------------------------------------------
if __name__ == '__main__':
    #test_choose_embedded()
    test_choose(False)



"""
summary: triggering bits of code by pressing a shortcut (older version)

description:
  This is a somewhat ancient way of registering actions & binding
  shortcuts. It's still here for reference, but "fresher" alternatives
  should be preferred.

keywords: actions

see_also: actions, add_hotkey
"""

import ida_expr
import ida_kernwin

def say_hi():
    print("Hotkey activated!")

# IDA binds hotkeys to IDC functions so a trampoline IDC function must be created
ida_expr.compile_idc_text('static key_2() { RunPythonStatement("say_hi()"); }')

# Add the hotkey
ida_kernwin.add_idc_hotkey("2", 'key_2')

# Press 2 to activate foo()

# The hotkey can be removed with
# ida_kernwin.del_idc_hotkey('2')




"""
summary: implement a "jump to next comment" action within IDA's disassembly view.

description:
  We want our action not only to find the next line containing a comment,
  but to also place the cursor at the right horizontal position.

  To find that position, we will have to inspect the text that IDA
  generates, looking for the start of a comment.
  However, we won't be looking for a comment "prefix" (e.g., "; "),
  as that would be too fragile.

  Instead, we will look for special "tags" that IDA injects into textual
  lines, and that bear semantic information.

  Those tags are primarily used for rendering (i.e., switching colors),
  but can also be very handy for spotting tokens of interest (registers,
  addresses, comments, prefixes, instruction mnemonics, ...)

keywords: idaview, actions

see_also: save_and_restore_listing_pos
"""

import ida_idaapi
import ida_kernwin
import ida_bytes
import ida_moves
import ida_lines

def find_comment_visual_position_in_tagged_line(line):
    """
    We'll look for tags for all types of comments, and if
    found return the visual position of the tag in the line
    (using 'ida_lines.tag_strlen')
    """
    for cmt_type in [
            ida_lines.SCOLOR_REGCMT,
            ida_lines.SCOLOR_RPTCMT,
            ida_lines.SCOLOR_AUTOCMT]:
        cmt_idx = line.find(ida_lines.SCOLOR_ON + cmt_type)
        if cmt_idx > -1:
            return ida_lines.tag_strlen(line[:cmt_idx])
    return -1


def jump_next_comment(v):
    """
    Starting at the current line, keep generating lines until
    a comment is found. When this happens, position the viewer
    at the right coordinates.
    """
    loc = ida_moves.lochist_entry_t()
    if ida_kernwin.get_custom_viewer_location(loc, v):
        place = loc.place()
        idaplace = ida_kernwin.place_t_as_idaplace_t(place)
        ea = idaplace.ea
        while ea != ida_idaapi.BADADDR:
            _, disass = ida_lines.generate_disassembly(
                ea,
                1000,  # maximum number of lines
                False, # as_stack=False
                False) # notags=False - we want tags, in order to spot comments

            found = None

            # If this is the start item, start at the next line
            start_lnnum = (idaplace.lnnum + 1) if ea == idaplace.ea else 0

            for rel_lnnum, line in enumerate(disass[start_lnnum:]):
                vis_cx = find_comment_visual_position_in_tagged_line(line)
                if vis_cx > -1:
                    found = (ea, rel_lnnum, vis_cx)
                    break

            if found is not None:
                idaplace.ea = found[0]
                idaplace.lnnum = start_lnnum + found[1]
                loc.set_place(idaplace)
                loc.renderer_info().pos.cx = found[2]
                ida_kernwin.custom_viewer_jump(v, loc, ida_kernwin.CVNF_LAZY)
                break

            ea = ida_bytes.next_head(ea, ida_idaapi.BADADDR)


class jump_next_comment_ah_t(ida_kernwin.action_handler_t):
    def activate(self, ctx):
        jump_next_comment(ctx.widget)

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ctx.widget_type == ida_kernwin.BWN_DISASM \
            else ida_kernwin.AST_DISABLE_FOR_WIDGET


ACTION_NAME = "jump_next_comment:jump"
ACTION_LABEL = "Jump to the next comment"
ACTION_SHORTCUT = "Ctrl+Alt+C"
ACTION_HELP = "Press %s to jump to the next comment" % ACTION_SHORTCUT

if ida_kernwin.unregister_action(ACTION_NAME):
    print("Unregistered previously-registered action \"%s\"" % ACTION_LABEL)

if ida_kernwin.register_action(
        ida_kernwin.action_desc_t(
            ACTION_NAME,
            ACTION_LABEL,
            jump_next_comment_ah_t(),
            ACTION_SHORTCUT)):
    print("Registered action \"%s\". %s" % (ACTION_LABEL, ACTION_HELP))




"""
summary: using `ida_hexrays.udc_filter_t`

description:
  Registers an action that uses a `ida_hexrays.udc_filter_t` to decompile
  `svc 0x900001` and `svc 0x9000F8` as function calls to
  `svc_exit()` and `svc_exit_group()` respectively.

  You will need to have an ARM + Linux IDB for this script to be usable

  In addition to having a shortcut, the action will be present
  in the context menu.
"""

import ida_idaapi
import ida_hexrays
import ida_kernwin
import ida_allins

ACTION_NAME = "vds8.py:udcall"
ACTION_SHORTCUT = "Ctrl+Shift+U"

# --------------------------------------------------------------------------
class udc_exit_t(ida_hexrays.udc_filter_t):
    def __init__(self, code, name):
        ida_hexrays.udc_filter_t.__init__(self)
        if not self.init("int __usercall %s@<R0>(int status@<R1>);" % name):
            raise Exception("Couldn't initialize udc_exit_t instance")
        self.code = code
        self.installed = False

    def match(self, cdg):
        return cdg.insn.itype == ida_allins.ARM_svc and cdg.insn.Op1.value == self.code

    def install(self):
        ida_hexrays.install_microcode_filter(self, True);
        self.installed = True

    def uninstall(self):
        ida_hexrays.install_microcode_filter(self, False);
        self.installed = False

    def toggle_install(self):
        if self.installed:
            self.uninstall()
        else:
            self.install()


# --------------------------------------------------------------------------
class toggle_udc_ah_t(ida_kernwin.action_handler_t):
    def __init__(self):
        ida_kernwin.action_handler_t.__init__(self)

    def activate(self, ctx):
        udc_exit.toggle_install();
        udc_exit_group.toggle_install();
        vu = ida_hexrays.get_widget_vdui(ctx.widget)
        vu.refresh_view(True)
        return 1

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET if \
            ctx.widget_type == ida_kernwin.BWN_PSEUDOCODE else \
            ida_kernwin.AST_DISABLE_FOR_WIDGET


# --------------------------------------------------------------------------
class my_hooks_t(ida_kernwin.UI_Hooks):
    def populating_widget_popup(self, widget, popup):
        if ida_kernwin.get_widget_type(widget) == ida_kernwin.BWN_PSEUDOCODE:
            ida_kernwin.attach_action_to_popup(widget, popup, ACTION_NAME)
my_hooks = my_hooks_t()
my_hooks.hook()


# --------------------------------------------------------------------------
SVC_EXIT       = 0x900001
SVC_EXIT_GROUP = 0x9000f8

if ida_hexrays.init_hexrays_plugin():
    udc_exit = udc_exit_t(SVC_EXIT, "svc_exit")
    udc_exit.toggle_install()

    udc_exit_group = udc_exit_t(SVC_EXIT_GROUP, "svc_exit_group")
    udc_exit_group.toggle_install()

    ida_kernwin.register_action(
        ida_kernwin.action_desc_t(
            ACTION_NAME,
            "vds8.py:Toggle UDC",
            toggle_udc_ah_t(),
            ACTION_SHORTCUT))




"""
summary: list bookmarks associated to a listing

description:
  This sample shows how to programmatically access the list of
  bookmarks placed in a listing widget (e.g., "IDA View-A",
  "Pseudocode-", …) using the low-level `ida_moves.bookmarks_t`
  type.

keywords: bookmarks
"""

import ida_kernwin
import ida_moves

class list_bookmarks_ah_t(ida_kernwin.action_handler_t):
    def activate(self, ctx):
        v = ida_kernwin.get_current_viewer()
        if v:
            print("### Bookmarks for %s" % ida_kernwin.get_widget_title(v))
            ud = ida_kernwin.get_viewer_user_data(v)
            for loc, desc in ida_moves.bookmarks_t(v):
                print("\t'%s': %s" % (loc.place()._print(ud), desc))

    def update(self, ctx):
        return ida_kernwin.AST_ENABLE_FOR_WIDGET \
            if ida_kernwin.get_current_viewer() \
            else ida_kernwin.AST_DISABLE_FOR_WIDGET

ACTION_NAME = "example:list_bookmarks"
ACTION_LABEL = "List bookmarks"
ACTION_SHORTCUT = "Ctrl+!"
ACTION_HELP = "Press %s to list bookmarks" % ACTION_SHORTCUT

if ida_kernwin.register_action(ida_kernwin.action_desc_t(
        ACTION_NAME,
        ACTION_LABEL,
        list_bookmarks_ah_t(),
        ACTION_SHORTCUT)):
    print("Registered action \"%s\". %s" % (ACTION_LABEL, ACTION_HELP))



"""
summary: dump user-defined information

description:
  Prints user-defined information to the "Output" window.
  Namely:

    * user defined label names
    * user defined indented comments
    * user defined number formats
    * user defined local variable names, types, comments

  This script loads information from the database without decompiling anything.

author: EiNSTeiN_ (einstein@g3nius.org)
"""

import ida_kernwin
import ida_hexrays
import ida_bytes

def run():
    f = ida_funcs.get_func(ida_kernwin.get_screen_ea());
    if f is None:
        print("Please position the cursor within a function")
        return True
    entry_ea = f.start_ea
    print("Dump of user-defined information for function at %x" % entry_ea)

    # Display user defined labels.
    labels = ida_hexrays.restore_user_labels(entry_ea);
    if labels is not None:
        print("------- %u user defined labels" % len(labels))
        for org_label, name in labels.items():
            print("Label %d: %s" % (org_label, str(name)))
        ida_hexrays.user_labels_free(labels)

    # Display user defined comments
    cmts = ida_hexrays.restore_user_cmts(entry_ea);
    if cmts is not None:
        print("------- %u user defined comments" % (len(cmts), ))
        for tl, cmt in cmts.items():
            print("Comment at %x, preciser %x:\n%s\n" % (tl.ea, tl.itp, str(cmt)))
        ida_hexrays.user_cmts_free(cmts)

    # Display user defined citem iflags
    iflags = ida_hexrays.restore_user_iflags(entry_ea)
    if iflags is not None:
        print("------- %u user defined citem iflags" % (len(iflags), ))
        for cl, f in iflags.items():
            print("%x(%d): %08X%s" % (cl.ea, cl.op, f, " CIT_COLLAPSED" if f & ida_hexrays.CIT_COLLAPSED else ""))
        ida_hexrays.user_iflags_free(iflags)

    # Display user defined number formats
    numforms = ida_hexrays.restore_user_numforms(entry_ea)
    if numforms is not None:
        print("------- %u user defined number formats" % (len(numforms), ))
        for ol, nf in numforms.items():
            print("Number format at %a, operand %d: %s" % \
                  (ol.ea,
                   ol.opnum,
                   "negated " if (ord(nf.props) & ida_hexrays.NF_NEGATE) != 0 else ""))

            if nf.is_enum():
                print("enum %s (serial %d)" % (str(nf.type_name), nf.serial))

            elif nf.is_char():
                print("char")

            elif nf.is_stroff():
                print("struct offset %s" % (str(nf.type_name), ))

            else:
                print("number base=%d" % (ida_bytes.get_radix(nf.flags, ol.opnum), ))

        ida_hexrays.user_numforms_free(numforms)

    # Display user-defined local variable information
    lvinf = ida_hexrays.lvar_uservec_t()
    if ida_hexrays.restore_user_lvar_settings(lvinf, entry_ea):
        print("------- User defined local variable information\n")
        for lv in lvinf.lvvec:
            print("Lvar defined at %x" % (lv.ll.defea, ))

            if len(str(lv.name)):
                print("  Name: %s" % (str(lv.name), ))

            if len(str(lv.type)):
                #~ print_type_to_one_line(buf, sizeof(buf), idati, .c_str());
                print("  Type: %s" % (str(lv.type), ))

            if len(str(lv.cmt)):
                print("  Comment: %s" % (str(lv.cmt), ))


    return


if ida_hexrays.init_hexrays_plugin():
    run()
else:
    print('dump user info: hexrays is not available.')



"""
summary: a custom microcode instruction optimization rule (`x | ~x => -1`)

description:
  Installs a custom microcode instruction optimization rule,
  to transform:

      x | ~x

  into

      -1

  To see this plugin in action please use be_ornot_be.idb
"""

import ida_hexrays
import ida_idaapi

# recognize "x | ~x" and replace by -1
class subinsn_optimizer_t(ida_hexrays.minsn_visitor_t):
    cnt = 0
    def visit_minsn(self):      # for each instruction...
        ins = self.curins       # take a reference to the current instruction

        # THE CORE OF THE PLUGIN IS HERE:
        # check the pattern "x | ~x"
        if ins.opcode == ida_hexrays.m_or and ins.r.is_insn(ida_hexrays.m_bnot) and ins.l == ins.r.d.l:
            if not ins.l.has_side_effects(): # avoid destroying side effects
                # pattern matched, convert to "mov -1, ..."
                ins.opcode = ida_hexrays.m_mov
                ins.l.make_number(-1, ins.r.size)
                ins.r = ida_hexrays.mop_t()
                self.cnt = self.cnt + 1 # number of changes we made
        return 0 # continue traversal

# a custom instruction optimizer, boilerplate code
class sample_optimizer_t(ida_hexrays.optinsn_t):
    def func(self, blk, ins, optflags):
        opt = subinsn_optimizer_t()
        ins.for_all_insns(opt)
        if opt.cnt != 0:                # if we modified microcode,
            blk.mba.verify(True)        # run the verifier
        return opt.cnt                  # report the number of changes

# a plugin interface, boilerplate code
class my_plugin_t(ida_idaapi.plugin_t):
    flags = ida_idaapi.PLUGIN_HIDE
    wanted_name = "optimize x|~x"
    wanted_hotkey = ""
    comment = ""
    help = ""
    def init(self):
        if ida_hexrays.init_hexrays_plugin():
            self.optimizer = sample_optimizer_t()
            self.optimizer.install()
            print("Installed sample optimizer for 'x | ~x'")
            return ida_idaapi.PLUGIN_KEEP # keep us in the memory
    def term(self):
        self.optimizer.remove()
    def run(self, arg):
        if arg == 1:
            return self.optimizer.remove()
        elif arg == 2:
            return self.optimizer.install()

def PLUGIN_ENTRY():
    return my_plugin_t()



"""
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ida_moves
else:
    import _ida_moves

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

SWIG_PYTHON_LEGACY_BOOL = _ida_moves.SWIG_PYTHON_LEGACY_BOOL

import ida_idaapi

class segm_move_info_vec_t(object):
    r"""
    Proxy of C++ qvector< segm_move_info_t > class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> segm_move_info_vec_t
        __init__(self, x) -> segm_move_info_vec_t

        @param x: qvector< segm_move_info_t > const &
        """
        _ida_moves.segm_move_info_vec_t_swiginit(self, _ida_moves.new_segm_move_info_vec_t(*args))
    __swig_destroy__ = _ida_moves.delete_segm_move_info_vec_t

    def push_back(self, *args) -> "segm_move_info_t &":
        r"""
        push_back(self, x)

        @param x: segm_move_info_t const &

        push_back(self) -> segm_move_info_t
        """
        return _ida_moves.segm_move_info_vec_t_push_back(self, *args)

    def pop_back(self, *args) -> "void":
        r"""
        pop_back(self)
        """
        return _ida_moves.segm_move_info_vec_t_pop_back(self, *args)

    def size(self, *args) -> "size_t":
        r"""
        size(self) -> size_t
        """
        return _ida_moves.segm_move_info_vec_t_size(self, *args)

    def empty(self, *args) -> "bool":
        r"""
        empty(self) -> bool
        """
        return _ida_moves.segm_move_info_vec_t_empty(self, *args)

    def at(self, *args) -> "segm_move_info_t const &":
        r"""
        at(self, _idx) -> segm_move_info_t

        @param _idx: size_t
        """
        return _ida_moves.segm_move_info_vec_t_at(self, *args)

    def qclear(self, *args) -> "void":
        r"""
        qclear(self)
        """
        return _ida_moves.segm_move_info_vec_t_qclear(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_moves.segm_move_info_vec_t_clear(self, *args)

    def resize(self, *args) -> "void":
        r"""
        resize(self, _newsize, x)

        @param _newsize: size_t
        @param x: segm_move_info_t const &

        resize(self, _newsize)

        @param _newsize: size_t
        """
        return _ida_moves.segm_move_info_vec_t_resize(self, *args)

    def grow(self, *args) -> "void":
        r"""
        grow(self, x=segm_move_info_t())

        @param x: segm_move_info_t const &
        """
        return _ida_moves.segm_move_info_vec_t_grow(self, *args)

    def capacity(self, *args) -> "size_t":
        r"""
        capacity(self) -> size_t
        """
        return _ida_moves.segm_move_info_vec_t_capacity(self, *args)

    def reserve(self, *args) -> "void":
        r"""
        reserve(self, cnt)

        @param cnt: size_t
        """
        return _ida_moves.segm_move_info_vec_t_reserve(self, *args)

    def truncate(self, *args) -> "void":
        r"""
        truncate(self)
        """
        return _ida_moves.segm_move_info_vec_t_truncate(self, *args)

    def swap(self, *args) -> "void":
        r"""
        swap(self, r)

        @param r: qvector< segm_move_info_t > &
        """
        return _ida_moves.segm_move_info_vec_t_swap(self, *args)

    def extract(self, *args) -> "segm_move_info_t *":
        r"""
        extract(self) -> segm_move_info_t
        """
        return _ida_moves.segm_move_info_vec_t_extract(self, *args)

    def inject(self, *args) -> "void":
        r"""
        inject(self, s, len)

        @param s: segm_move_info_t *
        @param len: size_t
        """
        return _ida_moves.segm_move_info_vec_t_inject(self, *args)

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: qvector< segm_move_info_t > const &
        """
        return _ida_moves.segm_move_info_vec_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: qvector< segm_move_info_t > const &
        """
        return _ida_moves.segm_move_info_vec_t___ne__(self, *args)

    def begin(self, *args) -> "qvector< segm_move_info_t >::const_iterator":
        r"""
        begin(self) -> segm_move_info_t
        """
        return _ida_moves.segm_move_info_vec_t_begin(self, *args)

    def end(self, *args) -> "qvector< segm_move_info_t >::const_iterator":
        r"""
        end(self) -> segm_move_info_t
        """
        return _ida_moves.segm_move_info_vec_t_end(self, *args)

    def insert(self, *args) -> "qvector< segm_move_info_t >::iterator":
        r"""
        insert(self, it, x) -> segm_move_info_t

        @param it: qvector< segm_move_info_t >::iterator
        @param x: segm_move_info_t const &
        """
        return _ida_moves.segm_move_info_vec_t_insert(self, *args)

    def erase(self, *args) -> "qvector< segm_move_info_t >::iterator":
        r"""
        erase(self, it) -> segm_move_info_t

        @param it: qvector< segm_move_info_t >::iterator

        erase(self, first, last) -> segm_move_info_t

        @param first: qvector< segm_move_info_t >::iterator
        @param last: qvector< segm_move_info_t >::iterator
        """
        return _ida_moves.segm_move_info_vec_t_erase(self, *args)

    def find(self, *args) -> "qvector< segm_move_info_t >::const_iterator":
        r"""
        find(self, x) -> segm_move_info_t

        @param x: segm_move_info_t const &

        """
        return _ida_moves.segm_move_info_vec_t_find(self, *args)

    def has(self, *args) -> "bool":
        r"""
        has(self, x) -> bool

        @param x: segm_move_info_t const &
        """
        return _ida_moves.segm_move_info_vec_t_has(self, *args)

    def add_unique(self, *args) -> "bool":
        r"""
        add_unique(self, x) -> bool

        @param x: segm_move_info_t const &
        """
        return _ida_moves.segm_move_info_vec_t_add_unique(self, *args)

    def _del(self, *args) -> "bool":
        r"""
        _del(self, x) -> bool

        Parameters
        ----------
        x: segm_move_info_t const &

        """
        return _ida_moves.segm_move_info_vec_t__del(self, *args)

    def __len__(self, *args) -> "size_t":
        r"""
        __len__(self) -> size_t
        """
        return _ida_moves.segm_move_info_vec_t___len__(self, *args)

    def __getitem__(self, *args) -> "segm_move_info_t const &":
        r"""
        __getitem__(self, i) -> segm_move_info_t

        @param i: size_t
        """
        return _ida_moves.segm_move_info_vec_t___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        r"""
        __setitem__(self, i, v)

        @param i: size_t
        @param v: segm_move_info_t const &
        """
        return _ida_moves.segm_move_info_vec_t___setitem__(self, *args)

    front = ida_idaapi._qvector_front
    back = ida_idaapi._qvector_back
    __iter__ = ida_idaapi._bounded_getitem_iterator


# Register segm_move_info_vec_t in _ida_moves:
_ida_moves.segm_move_info_vec_t_swigregister(segm_move_info_vec_t)


#<pycode(py_moves)>
import ida_kernwin
#</pycode(py_moves)>

class graph_location_info_t(object):
    r"""
    Proxy of C++ graph_location_info_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    zoom = property(_ida_moves.graph_location_info_t_zoom_get, _ida_moves.graph_location_info_t_zoom_set, doc=r"""zoom""")
    orgx = property(_ida_moves.graph_location_info_t_orgx_get, _ida_moves.graph_location_info_t_orgx_set, doc=r"""orgx""")
    orgy = property(_ida_moves.graph_location_info_t_orgy_get, _ida_moves.graph_location_info_t_orgy_set, doc=r"""orgy""")

    def __init__(self, *args):
        r"""
        __init__(self) -> graph_location_info_t
        """
        _ida_moves.graph_location_info_t_swiginit(self, _ida_moves.new_graph_location_info_t(*args))

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: graph_location_info_t const &
        """
        return _ida_moves.graph_location_info_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: graph_location_info_t const &
        """
        return _ida_moves.graph_location_info_t___ne__(self, *args)
    __swig_destroy__ = _ida_moves.delete_graph_location_info_t

# Register graph_location_info_t in _ida_moves:
_ida_moves.graph_location_info_t_swigregister(graph_location_info_t)

class segm_move_info_t(object):
    r"""
    Proxy of C++ segm_move_info_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self, _from=0, _to=0, _sz=0) -> segm_move_info_t

        @param _from: ea_t
        @param _to: ea_t
        @param _sz: size_t
        """
        _ida_moves.segm_move_info_t_swiginit(self, _ida_moves.new_segm_move_info_t(*args))
    _from = property(_ida_moves.segm_move_info_t__from_get, _ida_moves.segm_move_info_t__from_set, doc=r"""_from""")
    to = property(_ida_moves.segm_move_info_t_to_get, _ida_moves.segm_move_info_t_to_set, doc=r"""to""")
    size = property(_ida_moves.segm_move_info_t_size_get, _ida_moves.segm_move_info_t_size_set, doc=r"""size""")

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: segm_move_info_t const &
        """
        return _ida_moves.segm_move_info_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: segm_move_info_t const &
        """
        return _ida_moves.segm_move_info_t___ne__(self, *args)
    __swig_destroy__ = _ida_moves.delete_segm_move_info_t

# Register segm_move_info_t in _ida_moves:
_ida_moves.segm_move_info_t_swigregister(segm_move_info_t)

class segm_move_infos_t(segm_move_info_vec_t):
    r"""
    Proxy of C++ segm_move_infos_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def find(self, *args) -> "segm_move_info_t const *":
        r"""
        find(self, ea) -> segm_move_info_t

        @param ea: ea_t
        """
        return _ida_moves.segm_move_infos_t_find(self, *args)

    def __init__(self, *args):
        r"""
        __init__(self) -> segm_move_infos_t
        """
        _ida_moves.segm_move_infos_t_swiginit(self, _ida_moves.new_segm_move_infos_t(*args))
    __swig_destroy__ = _ida_moves.delete_segm_move_infos_t

# Register segm_move_infos_t in _ida_moves:
_ida_moves.segm_move_infos_t_swigregister(segm_move_infos_t)

class renderer_info_pos_t(object):
    r"""
    Proxy of C++ renderer_info_pos_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    node = property(_ida_moves.renderer_info_pos_t_node_get, _ida_moves.renderer_info_pos_t_node_set, doc=r"""node""")
    cx = property(_ida_moves.renderer_info_pos_t_cx_get, _ida_moves.renderer_info_pos_t_cx_set, doc=r"""cx""")
    cy = property(_ida_moves.renderer_info_pos_t_cy_get, _ida_moves.renderer_info_pos_t_cy_set, doc=r"""cy""")

    def __init__(self, *args):
        r"""
        __init__(self) -> renderer_info_pos_t
        """
        _ida_moves.renderer_info_pos_t_swiginit(self, _ida_moves.new_renderer_info_pos_t(*args))

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: renderer_info_pos_t const &
        """
        return _ida_moves.renderer_info_pos_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: renderer_info_pos_t const &
        """
        return _ida_moves.renderer_info_pos_t___ne__(self, *args)
    __swig_destroy__ = _ida_moves.delete_renderer_info_pos_t

# Register renderer_info_pos_t in _ida_moves:
_ida_moves.renderer_info_pos_t_swigregister(renderer_info_pos_t)

class renderer_info_t(object):
    r"""
    Proxy of C++ renderer_info_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    gli = property(_ida_moves.renderer_info_t_gli_get, _ida_moves.renderer_info_t_gli_set, doc=r"""gli""")
    pos = property(_ida_moves.renderer_info_t_pos_get, _ida_moves.renderer_info_t_pos_set, doc=r"""pos""")
    rtype = property(_ida_moves.renderer_info_t_rtype_get, _ida_moves.renderer_info_t_rtype_set, doc=r"""rtype""")

    def __init__(self, *args):
        r"""
        __init__(self) -> renderer_info_t
        __init__(self, _rtype, cx, cy) -> renderer_info_t

        @param _rtype: enum tcc_renderer_type_t
        @param cx: short
        @param cy: short
        """
        _ida_moves.renderer_info_t_swiginit(self, _ida_moves.new_renderer_info_t(*args))

    def __eq__(self, *args) -> "bool":
        r"""
        __eq__(self, r) -> bool

        @param r: renderer_info_t const &
        """
        return _ida_moves.renderer_info_t___eq__(self, *args)

    def __ne__(self, *args) -> "bool":
        r"""
        __ne__(self, r) -> bool

        @param r: renderer_info_t const &
        """
        return _ida_moves.renderer_info_t___ne__(self, *args)
    __swig_destroy__ = _ida_moves.delete_renderer_info_t

# Register renderer_info_t in _ida_moves:
_ida_moves.renderer_info_t_swigregister(renderer_info_t)

LSEF_PLACE = _ida_moves.LSEF_PLACE

LSEF_RINFO = _ida_moves.LSEF_RINFO

LSEF_PTYPE = _ida_moves.LSEF_PTYPE

LSEF_ALL = _ida_moves.LSEF_ALL

class lochist_entry_t(object):
    r"""
    Proxy of C++ lochist_entry_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    rinfo = property(_ida_moves.lochist_entry_t_rinfo_get, _ida_moves.lochist_entry_t_rinfo_set, doc=r"""rinfo""")
    plce = property(_ida_moves.lochist_entry_t_plce_get, _ida_moves.lochist_entry_t_plce_set, doc=r"""plce""")

    def __init__(self, *args):
        r"""
        __init__(self) -> lochist_entry_t
        __init__(self, p, r) -> lochist_entry_t

        @param p: place_t const *
        @param r: renderer_info_t const &

        __init__(self, other) -> lochist_entry_t

        @param other: lochist_entry_t const &
        """
        _ida_moves.lochist_entry_t_swiginit(self, _ida_moves.new_lochist_entry_t(*args))
    __swig_destroy__ = _ida_moves.delete_lochist_entry_t

    def renderer_info(self, *args) -> "renderer_info_t &":
        r"""
        renderer_info(self) -> renderer_info_t
        """
        return _ida_moves.lochist_entry_t_renderer_info(self, *args)

    def place(self, *args) -> "place_t *":
        r"""
        place(self) -> place_t
        """
        return _ida_moves.lochist_entry_t_place(self, *args)

    def set_place(self, *args) -> "void":
        r"""
        set_place(self, p)

        @param p: place_t const *
        """
        return _ida_moves.lochist_entry_t_set_place(self, *args)

    def is_valid(self, *args) -> "bool":
        r"""
        is_valid(self) -> bool
        """
        return _ida_moves.lochist_entry_t_is_valid(self, *args)

    def acquire_place(self, *args) -> "void":
        r"""
        acquire_place(self, in_p)

        @param in_p: place_t *
        """
        return _ida_moves.lochist_entry_t_acquire_place(self, *args)

# Register lochist_entry_t in _ida_moves:
_ida_moves.lochist_entry_t_swigregister(lochist_entry_t)

UNHID_SEGM = _ida_moves.UNHID_SEGM

UNHID_FUNC = _ida_moves.UNHID_FUNC

UNHID_RANGE = _ida_moves.UNHID_RANGE

DEFAULT_CURSOR_Y = _ida_moves.DEFAULT_CURSOR_Y

DEFAULT_LNNUM = _ida_moves.DEFAULT_LNNUM

CURLOC_LIST = _ida_moves.CURLOC_LIST

MAX_MARK_SLOT = _ida_moves.MAX_MARK_SLOT

class lochist_t(object):
    r"""
    Proxy of C++ lochist_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(self) -> lochist_t
        """
        _ida_moves.lochist_t_swiginit(self, _ida_moves.new_lochist_t(*args))
    __swig_destroy__ = _ida_moves.delete_lochist_t

    def is_history_enabled(self, *args) -> "bool":
        r"""
        is_history_enabled(self) -> bool
        """
        return _ida_moves.lochist_t_is_history_enabled(self, *args)

    def get_place_id(self, *args) -> "int":
        r"""
        get_place_id(self) -> int
        """
        return _ida_moves.lochist_t_get_place_id(self, *args)

    def init(self, *args) -> "bool":
        r"""
        init(self, stream_name, _defpos, _ud, _flags) -> bool

        @param stream_name: char const *
        @param _defpos: place_t const *
        @param _ud: void *
        @param _flags: uint32
        """
        return _ida_moves.lochist_t_init(self, *args)

    def netcode(self, *args) -> "nodeidx_t":
        r"""
        netcode(self) -> nodeidx_t
        """
        return _ida_moves.lochist_t_netcode(self, *args)

    def jump(self, *args) -> "void":
        r"""
        jump(self, try_to_unhide, e)

        @param try_to_unhide: bool
        @param e: lochist_entry_t const &
        """
        return _ida_moves.lochist_t_jump(self, *args)

    def current_index(self, *args) -> "uint32":
        r"""
        current_index(self) -> uint32
        """
        return _ida_moves.lochist_t_current_index(self, *args)

    def seek(self, *args) -> "bool":
        r"""
        seek(self, index, try_to_unhide) -> bool

        @param index: uint32
        @param try_to_unhide: bool
        """
        return _ida_moves.lochist_t_seek(self, *args)

    def fwd(self, *args) -> "bool":
        r"""
        fwd(self, cnt, try_to_unhide) -> bool

        @param cnt: uint32
        @param try_to_unhide: bool
        """
        return _ida_moves.lochist_t_fwd(self, *args)

    def back(self, *args) -> "bool":
        r"""
        back(self, cnt, try_to_unhide) -> bool

        @param cnt: uint32
        @param try_to_unhide: bool
        """
        return _ida_moves.lochist_t_back(self, *args)

    def save(self, *args) -> "void":
        r"""
        save(self)
        """
        return _ida_moves.lochist_t_save(self, *args)

    def clear(self, *args) -> "void":
        r"""
        clear(self)
        """
        return _ida_moves.lochist_t_clear(self, *args)

    def get_current(self, *args) -> "lochist_entry_t const &":
        r"""
        get_current(self) -> lochist_entry_t
        """
        return _ida_moves.lochist_t_get_current(self, *args)

    def set_current(self, *args) -> "void":
        r"""
        set_current(self, e)

        @param e: lochist_entry_t const &
        """
        return _ida_moves.lochist_t_set_current(self, *args)

    def set(self, *args) -> "void":
        r"""
        set(self, index, e)

        @param index: uint32
        @param e: lochist_entry_t const &
        """
        return _ida_moves.lochist_t_set(self, *args)

    def get(self, *args) -> "bool":
        r"""
        get(self, out, index) -> bool

        @param out: lochist_entry_t *
        @param index: uint32
        """
        return _ida_moves.lochist_t_get(self, *args)

    def size(self, *args) -> "uint32":
        r"""
        size(self) -> uint32
        """
        return _ida_moves.lochist_t_size(self, *args)

    def get_template_place(self, *args) -> "place_t const *":
        r"""
        get_template_place(self) -> place_t
        """
        return _ida_moves.lochist_t_get_template_place(self, *args)

# Register lochist_t in _ida_moves:
_ida_moves.lochist_t_swigregister(lochist_t)

class bookmarks_t(object):
    r"""
    Proxy of C++ bookmarks_t class.
    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def mark(*args) -> "uint32":
        r"""
        mark(e, index, title, desc, ud) -> uint32

        @param e: lochist_entry_t const &
        @param index: uint32
        @param title: char const *
        @param desc: char const *
        @param ud: void *
        """
        return _ida_moves.bookmarks_t_mark(*args)

    @staticmethod
    def get_desc(*args) -> "qstring *":
        r"""
        get_desc(e, index, ud) -> bool

        @param e: lochist_entry_t const &
        @param index: uint32
        @param ud: void *
        """
        return _ida_moves.bookmarks_t_get_desc(*args)

    @staticmethod
    def find_index(*args) -> "uint32":
        r"""
        find_index(e, ud) -> uint32

        @param e: lochist_entry_t const &
        @param ud: void *
        """
        return _ida_moves.bookmarks_t_find_index(*args)

    @staticmethod
    def size(*args) -> "uint32":
        r"""
        size(e, ud) -> uint32

        @param e: lochist_entry_t const &
        @param ud: void *
        """
        return _ida_moves.bookmarks_t_size(*args)

    @staticmethod
    def erase(*args) -> "bool":
        r"""
        erase(e, index, ud) -> bool

        @param e: lochist_entry_t const &
        @param index: uint32
        @param ud: void *
        """
        return _ida_moves.bookmarks_t_erase(*args)

    @staticmethod
    def get_dirtree_id(*args) -> "dirtree_id_t":
        r"""
        get_dirtree_id(e, ud) -> dirtree_id_t

        @param e: lochist_entry_t const &
        @param ud: void *
        """
        return _ida_moves.bookmarks_t_get_dirtree_id(*args)

    @staticmethod
    def get(*args) -> "PyObject *":
        r"""
        get(out, _index, ud) -> PyObject *

        @param out: lochist_entry_t *
        @param _index: uint32
        @param ud: void *
        """
        return _ida_moves.bookmarks_t_get(*args)

    def __init__(self, w):
        """
        Build an object suitable for iterating bookmarks
        associated with the specified widget.

        Note: all ea_t-based widgets (e.g., "IDA View-*",
        "Pseudocode-*", "Hex View-*", ...) share a common storage,
        so bookmarks can be re-used interchangeably between them
        """
        self.widget = w
        self.userdata = ida_kernwin.get_viewer_user_data(self.widget)
        self.template = lochist_entry_t()
        if ida_kernwin.get_custom_viewer_location(self.template, self.widget):
            p = self.template.place()
            if p is not None:
                p_id = ida_kernwin.get_place_class_id(p.name())
                if p_id > -1 and ida_kernwin.is_place_class_ea_capable(p_id):
                    idap_id = ida_kernwin.get_place_class_id("idaplace_t")
                    if idap_id > -1:
                        idap = ida_kernwin.get_place_class_template(idap_id)
                        if idap is not None:
                            self.template.set_place(idap)

    def __iter__(self):
        r"""
        Iterate on bookmarks present for the widget.
        """
        p = self.template.place()
        if p is not None:
            for idx in range(bookmarks_t.size(self.template, self.userdata)):
                yield self[idx]

    def __len__(self):
        r"""
        Get the number of bookmarks for the widget.
        """
        return bookmarks_t.size(self.template, self.userdata)

    def __getitem__(self, idx):
        r"""
        Get the n-th bookmark for the widget.
        """
        p = self.template.place()
        if p is not None:
            if isinstance(idx, int) and idx >= 0 and idx < len(self):
                loc = lochist_entry_t()
                loc.set_place(p)
                desc, _ = bookmarks_t.get(loc, idx, self.userdata)
                return loc, desc
            else:
                raise IndexError()


# Register bookmarks_t in _ida_moves:
_ida_moves.bookmarks_t_swigregister(bookmarks_t)
BOOKMARKS_PROMPT_WITH_HINT_PREFIX = _ida_moves.BOOKMARKS_PROMPT_WITH_HINT_PREFIX


def bookmarks_t_mark(*args) -> "uint32":
    r"""
    bookmarks_t_mark(e, index, title, desc, ud) -> uint32

    @param e: lochist_entry_t const &
    @param index: uint32
    @param title: char const *
    @param desc: char const *
    @param ud: void *
    """
    return _ida_moves.bookmarks_t_mark(*args)

def bookmarks_t_get_desc(*args) -> "qstring *":
    r"""
    bookmarks_t_get_desc(e, index, ud) -> str

    @param e: lochist_entry_t const &
    @param index: uint32
    @param ud: void *
    """
    return _ida_moves.bookmarks_t_get_desc(*args)

def bookmarks_t_find_index(*args) -> "uint32":
    r"""
    bookmarks_t_find_index(e, ud) -> uint32

    @param e: lochist_entry_t const &
    @param ud: void *
    """
    return _ida_moves.bookmarks_t_find_index(*args)

def bookmarks_t_size(*args) -> "uint32":
    r"""
    bookmarks_t_size(e, ud) -> uint32

    @param e: lochist_entry_t const &
    @param ud: void *
    """
    return _ida_moves.bookmarks_t_size(*args)

def bookmarks_t_erase(*args) -> "bool":
    r"""
    bookmarks_t_erase(e, index, ud) -> bool

    @param e: lochist_entry_t const &
    @param index: uint32
    @param ud: void *
    """
    return _ida_moves.bookmarks_t_erase(*args)

def bookmarks_t_get_dirtree_id(*args) -> "dirtree_id_t":
    r"""
    bookmarks_t_get_dirtree_id(e, ud) -> dirtree_id_t

    @param e: lochist_entry_t const &
    @param ud: void *
    """
    return _ida_moves.bookmarks_t_get_dirtree_id(*args)

def bookmarks_t_get(*args) -> "PyObject *":
    r"""
    bookmarks_t_get(out, _index, ud) -> PyObject *

    @param out: lochist_entry_t *
    @param _index: uint32
    @param ud: void *
    """
    return _ida_moves.bookmarks_t_get(*args)







"""
summary: modifying local variables

description:
  Use a `ida_hexrays.user_lvar_modifier_t` to modify names,
  comments and/or types of local variables.
"""

import ida_hexrays
import ida_typeinf

import idc

class my_modifier_t(ida_hexrays.user_lvar_modifier_t):
    def __init__(self, name_prefix="", cmt_prefix="", new_types={}):
        ida_hexrays.user_lvar_modifier_t.__init__(self)
        self.name_prefix = name_prefix
        self.cmt_prefix = cmt_prefix
        self.new_types = new_types

    def modify_lvars(self, lvars):
        def log(msg):
            print("modify_lvars: %s" % msg)
        log("len(lvars.lvvec) = %d" % len(lvars.lvvec))
        log("lvars.lmaps.size() = %d" % lvars.lmaps.size())
        log("lvars.stkoff_delta = %d" % lvars.stkoff_delta)
        log("lvars.ulv_flags = %x" % lvars.ulv_flags)

        for idx, one in enumerate(lvars.lvvec):
            def varlog(msg):
                log("var #%d: %s" % (idx, msg))
            varlog("name = '%s'" % one.name)
            varlog("type = '%s'" % one.type._print())
            varlog("cmt = '%s'" % one.cmt)
            varlog("size = %d" % one.size)
            varlog("flags = %x" % one.flags)
            new_type = self.new_types.get(one.name)
            if new_type:
                tif = ida_typeinf.tinfo_t()
                ida_typeinf.parse_decl(tif, None, new_type, 0)
                one.type = tif
            one.name = self.name_prefix + one.name
            one.cmt = self.cmt_prefix + one.cmt

        return True


def modify_function_lvars(name_prefix="patched_", cmt_prefix="(patched) ", new_types={}):
    ea = idc.here()
    my_mod = my_modifier_t(
        name_prefix=name_prefix,
        cmt_prefix=cmt_prefix,
        new_types=new_types)
    ida_hexrays.modify_user_lvars(ea, my_mod)



"""
summary: show decompiler xrefs

description:
  Show decompiler-style Xref when the `Ctrl+X` key is
  pressed in the Decompiler window.

  * supports any global name: functions, strings, integers, ...
  * supports structure member.

author: EiNSTeiN_ (einstein@g3nius.org)
"""

import ida_kernwin
import ida_hexrays
import ida_typeinf
import ida_idaapi
import ida_struct
import ida_funcs

import idautils

import traceback

from PyQt5 import QtCore, QtWidgets

XREF_EA = 0
XREF_STRUC_MEMBER = 1

class XrefsForm(ida_kernwin.PluginForm):

    def __init__(self, target):

        ida_kernwin.PluginForm.__init__(self)

        self.target = target

        if type(self.target) == ida_hexrays.cfunc_t:

            self.__type = XREF_EA
            self.__ea = self.target.entry_ea
            self.__name = 'Xrefs of %x' % (self.__ea, )

        elif type(self.target) == ida_hexrays.cexpr_t and self.target.opname == 'obj':

            self.__type = XREF_EA
            self.__ea = self.target.obj_ea
            self.__name = 'Xrefs of %x' % (self.__ea, )

        elif type(self.target) == ida_hexrays.cexpr_t and self.target.opname in ('memptr', 'memref'):

            self.__type = XREF_STRUC_MEMBER
            name = self.get_struc_name()
            self.__name = 'Xrefs of %s' % (name, )

        else:
            raise ValueError('cannot show xrefs for this kind of target')

        return

    def get_struc_name(self):

        x = self.target.operands['x']
        m = self.target.operands['m']

        xtype = x.type
        xtype.remove_ptr_or_array()
        typename = ida_typeinf.print_tinfo('', 0, 0, ida_typeinf.PRTYPE_1LINE, xtype, '', '')

        sid = ida_struct.get_struc_id(typename)
        sptr = ida_struct.get_struc(sid)
        member = ida_struct.get_member(sptr, m)

        return '%s::%s' % (typename, member)

    def OnCreate(self, widget):

        # Get parent widget
        self.parent = self.FormToPyQtWidget(widget)

        self.populate_form()

        return

    def Show(self):
        ida_kernwin.PluginForm.Show(self, self.__name)
        return

    def populate_form(self):
        # Create layout
        layout = QtWidgets.QVBoxLayout()

        layout.addWidget(QtWidgets.QLabel(self.__name))
        self.table = QtWidgets.QTableWidget()
        layout.addWidget(self.table)

        self.table.setColumnCount(3)
        self.table.setHorizontalHeaderItem(0, QtWidgets.QTableWidgetItem("Address"))
        self.table.setHorizontalHeaderItem(1, QtWidgets.QTableWidgetItem("Function"))
        self.table.setHorizontalHeaderItem(2, QtWidgets.QTableWidgetItem("Line"))

        self.table.setColumnWidth(0, 80)
        self.table.setColumnWidth(1, 150)
        self.table.setColumnWidth(2, 450)

        self.table.cellDoubleClicked.connect(self.double_clicked)

        #~ self.table.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows )
        self.parent.setLayout(layout)

        self.populate_table()

        return

    def double_clicked(self, row, column):

        ea = self.functions[row]
        ida_hexrays.open_pseudocode(ea, True)

        return

    def get_decompiled_line(self, cfunc, ea):

        print(repr(ea))
        if ea not in cfunc.eamap:
            print('strange, %x is not in %x eamap' % (ea, cfunc.entry_ea))
            return

        insnvec = cfunc.eamap[ea]

        lines = []
        for stmt in insnvec:

            qp = ida_hexrays.qstring_printer_t(cfunc, False)

            stmt._print(0, qp)
            s = qp.s.split('\n')[0]

            #~ s = ida_lines.tag_remove(s)
            lines.append(s)

        return '\n'.join(lines)

    def get_items_for_ea(self, ea):

        frm = [x.frm for x in idautils.XrefsTo(self.__ea)]

        items = []
        for ea in frm:
            cfunc = ida_hexrays.decompile(ea)
            if not cfunc:
                print('Decompilation of %x failed' % (ea, ))
                continue

            self.functions.append(cfunc.entry_ea)
            self.items.append((ea, ida_funcs.get_func_name(cfunc.entry_ea) or "", self.get_decompiled_line(cfunc, ea)))
    def get_items_for_type(self):

        x = self.target.operands['x']
        m = self.target.operands['m']

        xtype = x.type
        xtype.remove_ptr_or_array()
        typename = ida_typeinf.print_tinfo('', 0, 0, ida_typeinf.PRTYPE_1LINE, xtype, '', '')

        addresses = []
        for ea in idautils.Functions():

            cfunc = ida_hexrays.decompile(ea, flags=ida_hexrays.DECOMP_GXREFS_FORCE)
            if not cfunc:
                print('Decompilation of %x failed' % (ea, ))
                continue

            for citem in cfunc.treeitems:
                citem = citem.to_specific_type
                if not (type(citem) == ida_hexrays.cexpr_t and citem.opname in ('memptr', 'memref')):
                    continue

                _x = citem.operands['x']
                _m = citem.operands['m']
                _xtype = _x.type
                _xtype.remove_ptr_or_array()
                _typename = ida_typeinf.print_tinfo('', 0, 0, ida_typeinf.PRTYPE_1LINE, _xtype, '', '')

                if not (_typename == typename and _m == m):
                    continue

                parent = citem
                while parent:
                    if type(parent.to_specific_type) == ida_hexrays.cinsn_t:
                        break
                    parent = cfunc.body.find_parent_of(parent)

                if not parent:
                    print('cannot find parent statement (?!)')
                    continue

                if parent.ea in addresses:
                    continue

                if parent.ea == ida_idaapi.BADADDR:
                    print('parent.ea is BADADDR')
                    continue

                addresses.append(parent.ea)

                self.functions.append(cfunc.entry_ea)
                self.items.append((
                        parent.ea,
                        ida_funcs.get_func_name(cfunc.entry_ea) or "",
                        self.get_decompiled_line(cfunc, parent.ea)))


        return []

    def populate_table(self):

        self.functions = []
        self.items = []

        if self.__type == XREF_EA:
            self.get_items_for_ea(self.__ea)
        else:
            self.get_items_for_type()

        self.table.setRowCount(len(self.items))

        i = 0
        for item in self.items:
            address, func, line = item
            item = QtWidgets.QTableWidgetItem('0x%x' % (address, ))
            item.setFlags(item.flags() ^ QtCore.Qt.ItemIsEditable)
            self.table.setItem(i, 0, item)
            item = QtWidgets.QTableWidgetItem(func)
            item.setFlags(item.flags() ^ QtCore.Qt.ItemIsEditable)
            self.table.setItem(i, 1, item)
            item = QtWidgets.QTableWidgetItem(line)
            item.setFlags(item.flags() ^ QtCore.Qt.ItemIsEditable)
            self.table.setItem(i, 2, item)

            i += 1

        self.table.resizeRowsToContents()

        return

    def OnClose(self, widget):
        pass


class show_xrefs_ah_t(ida_kernwin.action_handler_t):
    def __init__(self):
        ida_kernwin.action_handler_t.__init__(self)
        self.sel = None

    def activate(self, ctx):
        vu = ida_hexrays.get_widget_vdui(ctx.widget)
        if not vu or not self.sel:
            print("No vdui? Strange, since this action should be enabled only for pseudocode views.")
            return 0

        form = XrefsForm(self.sel)
        form.Show()
        return 1

    def update(self, ctx):
        if ctx.widget_type != ida_kernwin.BWN_PSEUDOCODE:
            return ida_kernwin.AST_DISABLE_FOR_WIDGET
        vu = ida_hexrays.get_widget_vdui(ctx.widget)
        vu.get_current_item(ida_hexrays.USE_KEYBOARD)
        item = vu.item
        self.sel = None
        if item.citype == ida_hexrays.VDI_EXPR and item.it.to_specific_type.opname in ('obj', 'memref', 'memptr'):
            # if an expression is selected. verify that it's either a cot_obj, cot_memref or cot_memptr
            self.sel = item.it.to_specific_type

        elif item.citype == ida_hexrays.VDI_FUNC:
            # if the function itself is selected, show xrefs to it.
            self.sel = item.f

        return ida_kernwin.AST_ENABLE if self.sel else ida_kernwin.AST_DISABLE


class vds_xrefs_hooks_t(ida_hexrays.Hexrays_Hooks):
    def populating_popup(self, widget, phandle, vu):
        ida_kernwin.attach_action_to_popup(widget, phandle, "vdsxrefs:show", None)
        return 0


if ida_hexrays.init_hexrays_plugin():
    adesc = ida_kernwin.action_desc_t('vdsxrefs:show', 'Show xrefs', show_xrefs_ah_t(), "Ctrl+X")
    if ida_kernwin.register_action(adesc):
        vds_xrefs_hooks = vds_xrefs_hooks_t()
        vds_xrefs_hooks.hook()
    else:
        print("Couldn't register action.")
else:
    print('hexrays is not available.')
